@@ -92,8 +92,7 @@ regex_make_shared = re.compile(r"std::make_shared<")
 
 # Contains per-file exceptions to violations of "make_shared".
 make_shared_exceptions = {
-    "*": ["tdb::make_shared"],
-    "*": ["tiledb::common::make_shared"],
+    "*": ["tdb::make_shared", "tiledb::common::make_shared"],
 }
 
 # Match C++ unique_ptr objects.
@@ -103,7 +102,7 @@ regex_unique_ptr = re.compile(r"unique_ptr<")
 unique_ptr_exceptions = {
     "*": ["tdb_unique_ptr", "tiledb_unique_ptr", "tdb::pmr::unique_ptr"],
     "zstd_compressor.h": ["std::unique_ptr<ZSTD_DCtx, decltype(&ZSTD_freeDCtx)> ctx_;", "std::unique_ptr<ZSTD_CCtx, decltype(&ZSTD_freeCCtx)> ctx_;"],
-    "posix.cc": ["static std::unique_ptr<char, decltype(&free)> cwd_(getcwd(nullptr, 0), free);"],
+    "posix.cc": ["std::unique_ptr<DIR, UniqueDIRDeleter>", "static std::unique_ptr<char, decltype(&free)> cwd_(getcwd(nullptr, 0), free);"],
     "curl.h": ["std::unique_ptr<CURL, decltype(&curl_easy_cleanup)>"],
     "pmr.h": ["std::unique_ptr", "unique_ptr<Tp> make_unique("],
 }
@@ -148,7 +147,7 @@ violation_checkers = [
 
 
 def iter_file_violations(file_path: str) -> Iterable[Violation]:
-    with open(file_path) as f:
+    with open(file_path, encoding="utf-8") as f:
         for line_num, line in enumerate(f):
             line = line.strip()
             for checker in violation_checkers:
