@@ -33,7 +33,9 @@
 #include "test/support/src/helpers.h"
 #include "tiledb/api/c_api/buffer/buffer_api_internal.h"
 #include "tiledb/api/c_api/config/config_api_internal.h"
+#include "tiledb/api/c_api/context/context_api_internal.h"
 #include "tiledb/sm/c_api/tiledb_serialization.h"
+#include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/cpp_api/tiledb_experimental"
 #include "tiledb/sm/enums/serialization_type.h"
@@ -54,21 +56,17 @@ struct CppConsolidationPlanFx {
   VFS vfs_;
   Config cfg_;
 
-  std::string key_ = "0123456789abcdeF0123456789abcdeF";
-  const tiledb_encryption_type_t enc_type_ = TILEDB_AES_256_GCM;
-
   // Constructors/destructors.
   CppConsolidationPlanFx();
   ~CppConsolidationPlanFx();
 
   // Functions.
-  void create_sparse_array(bool allows_dups = false, bool encrypt = false);
+  void create_sparse_array(bool allows_dups = false);
   std::string write_sparse(
       std::vector<int> a1,
       std::vector<uint64_t> dim1,
       std::vector<uint64_t> dim2,
-      uint64_t timestamp,
-      bool encrypt = false);
+      uint64_t timestamp);
   void remove_sparse_array();
   void remove_array(const std::string& array_name);
   bool is_array(const std::string& array_name);
@@ -97,8 +95,7 @@ CppConsolidationPlanFx::~CppConsolidationPlanFx() {
   remove_sparse_array();
 }
 
-void CppConsolidationPlanFx::create_sparse_array(
-    bool allows_dups, bool encrypt) {
+void CppConsolidationPlanFx::create_sparse_array(bool allows_dups) {
   // Create dimensions.
   auto d1 = Dimension::create<uint64_t>(ctx_, "d1", {{1, 999}}, 2);
   auto d2 = Dimension::create<uint64_t>(ctx_, "d2", {{1, 999}}, 2);
@@ -127,35 +124,21 @@ void CppConsolidationPlanFx::create_sparse_array(
   filter_list.add_filter(filter);
   schema.set_coords_filter_list(filter_list);
 
-  if (encrypt) {
-    Array::create(SPARSE_ARRAY_NAME, schema, enc_type_, key_);
-  } else {
-    Array::create(SPARSE_ARRAY_NAME, schema);
-  }
+  Array::create(SPARSE_ARRAY_NAME, schema);
 }
 
 std::string CppConsolidationPlanFx::write_sparse(
     std::vector<int> a1,
     std::vector<uint64_t> dim1,
     std::vector<uint64_t> dim2,
-    uint64_t timestamp,
-    bool encrypt) {
+    uint64_t timestamp) {
   // Open array.
   std::unique_ptr<Array> array;
-  if (encrypt) {
-    array = std::make_unique<Array>(
-        ctx_,
-        SPARSE_ARRAY_NAME,
-        TILEDB_WRITE,
-        TemporalPolicy(TimeTravel, timestamp),
-        EncryptionAlgorithm(AESGCM, key_.c_str()));
-  } else {
-    array = std::make_unique<Array>(
-        ctx_,
-        SPARSE_ARRAY_NAME,
-        TILEDB_WRITE,
-        TemporalPolicy(TimestampStartEnd, 0, timestamp));
-  }
+  array = std::make_unique<Array>(
+      ctx_,
+      SPARSE_ARRAY_NAME,
+      TILEDB_WRITE,
+      TemporalPolicy(TimestampStartEnd, 0, timestamp));
 
   // Create query.
   Query query(ctx_, *array, TILEDB_WRITE);
@@ -204,8 +187,10 @@ tiledb::sm::ConsolidationPlan CppConsolidationPlanFx::call_handler(
     uint64_t fragment_size,
     const Array& array,
     tiledb::sm::SerializationType stype) {
-  auto req_buf = tiledb_buffer_handle_t::make_handle();
-  auto resp_buf = tiledb_buffer_handle_t::make_handle();
+  auto req_buf = tiledb_buffer_handle_t::make_handle(
+      ctx_.ptr().get()->resources().serialization_memory_tracker());
+  auto resp_buf = tiledb_buffer_handle_t::make_handle(
+      ctx_.ptr().get()->resources().serialization_memory_tracker());
 
   tiledb::sm::serialization::serialize_consolidation_plan_request(
       fragment_size, cfg_.ptr()->config(), stype, req_buf->buffer());
