@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,7 +32,7 @@
 
 #include "test/support/src/helpers.h"
 #include "test/support/src/vfs_helpers.h"
-#include "tiledb/sm/c_api/tiledb_struct_def.h"
+#include "tiledb/api/c_api/array/array_api_internal.h"
 #include "tiledb/sm/subarray/subarray_partitioner.h"
 
 #ifdef _WIN32
@@ -124,7 +124,7 @@ TEST_CASE_METHOD(
   Subarray subarray;
   SubarrayRanges<uint64_t> ranges = {{5, 7, 6, 15, 33, 43}};
   Layout subarray_layout = Layout::ROW_MAJOR;
-  create_subarray(array_->array_, ranges, subarray_layout, &subarray);
+  create_subarray(array_->array(), ranges, subarray_layout, &subarray);
   CHECK_NOTHROW(subarray.compute_tile_coords<uint64_t>());
 
   // Prepare correct tile coordinates
@@ -247,7 +247,7 @@ TEST_CASE_METHOD(
   Subarray subarray;
   SubarrayRanges<uint64_t> ranges = {{2, 2, 6, 10}, {2, 6, 5, 10}};
   Layout subarray_layout = Layout::ROW_MAJOR;
-  create_subarray(array_->array_, ranges, subarray_layout, &subarray);
+  create_subarray(array_->array(), ranges, subarray_layout, &subarray);
   CHECK_NOTHROW(subarray.compute_tile_coords<uint64_t>());
 
   auto tile_coords = subarray.tile_coords();
@@ -300,7 +300,7 @@ TEST_CASE_METHOD(
   Subarray subarray;
   SubarrayRanges<uint64_t> ranges = {{2, 10, 6, 10}, {2, 6, 5, 10}};
   Layout subarray_layout = Layout::ROW_MAJOR;
-  create_subarray(array_->array_, ranges, subarray_layout, &subarray);
+  create_subarray(array_->array(), ranges, subarray_layout, &subarray);
 
   std::vector<uint64_t> tile_coords = {1, 0};
   std::vector<uint64_t> c_range_0_0 = {3, 4};
@@ -317,6 +317,9 @@ TEST_CASE_METHOD(
   CHECK_NOTHROW(cropped_subarray.get_range(1, 1, &range));
   CHECK(!memcmp(range->data(), &c_range_1_1[0], 2 * sizeof(uint64_t)));
 
+  auto tile_cell_num = subarray.tile_cell_num(&tile_coords[0]);
+  CHECK(tile_cell_num == cropped_subarray.cell_num());
+
   close_array(ctx_, array_);
 }
 
@@ -407,7 +410,7 @@ TEST_CASE_METHOD(
   }
   SubarrayRanges<uint64_t> ranges = {d1_ranges, d2_ranges};
   Layout subarray_layout = Layout::ROW_MAJOR;
-  create_subarray(array_->array_, ranges, subarray_layout, &subarray, false);
+  create_subarray(array_->array(), ranges, subarray_layout, &subarray, false);
 
   // We must compute range offsets before invoking
   // `get_expanded_coordinates`.
@@ -482,7 +485,7 @@ TEST_CASE_METHOD(
   }
   SubarrayRanges<uint64_t> ranges = {d1_ranges, d2_ranges};
   Layout subarray_layout = Layout::COL_MAJOR;
-  create_subarray(array_->array_, ranges, subarray_layout, &subarray, false);
+  create_subarray(array_->array(), ranges, subarray_layout, &subarray, false);
 
   // We must compute range offsets before invoking
   // `get_expanded_coordinates`.
@@ -557,7 +560,7 @@ TEST_CASE_METHOD(
   }
   SubarrayRanges<uint64_t> ranges = {d1_ranges, d2_ranges};
   Layout subarray_layout = Layout::UNORDERED;
-  create_subarray(array_->array_, ranges, subarray_layout, &subarray, false);
+  create_subarray(array_->array(), ranges, subarray_layout, &subarray, false);
 
   // We must compute range offsets before invoking
   // `get_expanded_coordinates`.
@@ -701,7 +704,7 @@ TEST_CASE_METHOD(
   }
   SubarrayRanges<uint64_t> ranges = {d1_ranges, d2_ranges, d3_ranges};
   Layout subarray_layout = Layout::ROW_MAJOR;
-  create_subarray(array_->array_, ranges, subarray_layout, &subarray, false);
+  create_subarray(array_->array(), ranges, subarray_layout, &subarray, false);
 
   // We must compute range offsets before invoking
   // `get_expanded_coordinates`.
@@ -800,7 +803,7 @@ TEST_CASE_METHOD(
   }
   SubarrayRanges<uint64_t> ranges = {d1_ranges, d2_ranges, d3_ranges};
   Layout subarray_layout = Layout::COL_MAJOR;
-  create_subarray(array_->array_, ranges, subarray_layout, &subarray, false);
+  create_subarray(array_->array(), ranges, subarray_layout, &subarray, false);
 
   // We must compute range offsets before invoking
   // `get_expanded_coordinates`.
@@ -899,7 +902,7 @@ TEST_CASE_METHOD(
   }
   SubarrayRanges<uint64_t> ranges = {d1_ranges, d2_ranges, d3_ranges};
   Layout subarray_layout = Layout::UNORDERED;
-  create_subarray(array_->array_, ranges, subarray_layout, &subarray, false);
+  create_subarray(array_->array(), ranges, subarray_layout, &subarray, false);
 
   // We must compute range offsets before invoking
   // `get_expanded_coordinates`.
@@ -959,10 +962,10 @@ TEST_CASE_METHOD(
       Range(&range_data[0], &range_data[1], sizeof(int64_t)),
       Range(&range_data[2], &range_data[3], sizeof(int64_t)),
       Range(&range_data[4], &range_data[5], sizeof(int64_t))};
-  subarray->subarray_->set_attribute_ranges("b", input_ranges);
+  subarray->set_attribute_ranges("b", input_ranges);
 
   // Get attribute ranges and verify results
-  const auto& output_ranges = subarray->subarray_->get_attribute_ranges("b");
+  const auto& output_ranges = subarray->get_attribute_ranges("b");
   for (uint32_t ii = 0; ii < 3; ++ii) {
     CHECK(input_ranges[ii] == output_ranges[ii]);
   }
