@@ -39,7 +39,6 @@
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/filesystem/uri.h"
 #include "tiledb/sm/misc/constants.h"
-#include "tiledb/sm/misc/utils.h"
 
 using namespace tiledb;
 
@@ -76,7 +75,7 @@ struct CPPArrayFx {
     schema.add_attributes(a1, a2, a3, a4, a5);
 
     // set the array_uri so that it's deleted on cleanup
-    Array::create(array_uri_, schema);
+    Array::create(ctx, array_uri_, schema);
   }
 
   test::VFSTestSetup vfs_test_setup_;
@@ -136,7 +135,7 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic][rest]") {
     schema.set_domain(domain);
     schema.add_attribute(a1);
 
-    Array::create(array_uri1, schema);
+    Array::create(ctx, array_uri1, schema);
     Array array(ctx1, array_uri1, TILEDB_READ);
 
     // Check that the config values are correct
@@ -197,7 +196,9 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic][rest]") {
 
     Query query(ctx, array, TILEDB_WRITE);
     CHECK(query.query_type() == TILEDB_WRITE);
-    query.set_subarray(subarray);
+    Subarray sub(ctx, array);
+    sub.set_subarray(subarray);
+    query.set_subarray(sub);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2buf.second);
     query.set_offsets_buffer("a2", a2buf.first);
@@ -254,6 +255,8 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic][rest]") {
       a1.resize(48);
 
       Query query(ctx, array);
+      Subarray sub(ctx, array);
+      sub.set_subarray(subarray);
 
       CHECK(!query.has_results());
 
@@ -265,7 +268,7 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic][rest]") {
       query.set_offsets_buffer("a4", a4buf.first);
       query.set_data_buffer("a5", a5);
       query.set_layout(TILEDB_ROW_MAJOR);
-      query.set_subarray(subarray);
+      query.set_subarray(sub);
 
       // Make sure no segfault when called before submit
       query.result_buffer_elements();
@@ -346,7 +349,9 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic][rest]") {
 
     Array array(ctx, array_uri_, TILEDB_WRITE);
     Query query(ctx, array, TILEDB_WRITE);
-    query.set_subarray(subarray);
+    Subarray sub(ctx, array);
+    sub.set_subarray(subarray);
+    query.set_subarray(sub);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2buf.second);
     query.set_offsets_buffer("a2", a2buf.first);
@@ -405,7 +410,9 @@ TEST_CASE_METHOD(CPPArrayFx, "C++ API: Arrays", "[cppapi][basic][rest]") {
     std::vector<int> subarray = {0, 1, 0, 0};
     Array array(ctx, array_uri_, TILEDB_WRITE);
     Query query(ctx, array, TILEDB_WRITE);
-    query.set_subarray(subarray);
+    Subarray sub(ctx, array);
+    sub.set_subarray(subarray);
+    query.set_subarray(sub);
     query.set_data_buffer("a1", a1);
     query.set_layout(TILEDB_GLOBAL_ORDER);
     // Incorrect subarray for global order
@@ -466,7 +473,7 @@ TEST_CASE(
   schema.set_domain(domain);
   schema.add_attribute(Attribute::create<std::vector<int32_t>>(ctx, "a"));
   schema.add_attribute(Attribute::create<uint64_t>(ctx, "b"));
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   {
     Array array(ctx, array_uri, TILEDB_WRITE);
@@ -509,8 +516,10 @@ TEST_CASE(
     a = {};
     const std::vector<int> subarray = {0, 2};
     Query q(ctx, array, TILEDB_READ);
+    Subarray s(ctx, array);
+    s.set_subarray(subarray);
     q.set_layout(TILEDB_GLOBAL_ORDER);
-    q.set_subarray(subarray);
+    q.set_subarray(s);
     q.set_data_buffer("a", a);
     q.set_offsets_buffer("a", a_offset);
     q.set_data_buffer("b", b);
@@ -539,7 +548,7 @@ TEST_CASE("C++ API: Incorrect offsets", "[cppapi][invalid-offsets][rest]") {
   domain.add_dimension(Dimension::create<int32_t>(ctx, "d", {{0, 1000}}, 1001));
   schema.set_domain(domain);
   schema.add_attribute(Attribute::create<std::vector<int32_t>>(ctx, "a"));
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
   Array array(ctx, array_uri, TILEDB_WRITE);
 
   std::vector<int32_t> a, coord = {10, 20, 30};
@@ -585,7 +594,7 @@ TEST_CASE(
         ArraySchema schema(ctx, TILEDB_DENSE);
         schema.set_domain(domain).set_order({{tile_layout, cell_layout}});
         schema.add_attribute(Attribute::create<int>(ctx, "a"));
-        Array::create(array_name, schema);
+        Array::create(ctx, array_name, schema);
 
         // Write
         std::vector<int> data_w = {
@@ -603,7 +612,9 @@ TEST_CASE(
         Query query(ctx, array);
         const std::vector<int> subarray = {0, 3, 0, 3};
         std::vector<int> data(16);
-        query.set_subarray(subarray)
+        Subarray sub(ctx, array);
+        sub.set_subarray(subarray);
+        query.set_subarray(sub)
             .set_layout(TILEDB_ROW_MAJOR)
             .set_data_buffer("a", data);
         query.submit();
@@ -639,7 +650,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_DENSE);
   schema.set_domain(domain);
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
-  Array::create(array_name, schema);
+  Array::create(ctx, array_name, schema);
 
   REQUIRE_NOTHROW(Array::consolidate(ctx, array_name));
 
@@ -651,7 +662,10 @@ TEST_CASE(
     "C++ API: Consolidation of sequential fragment writes",
     "[cppapi][consolidation][sequential][non-rest]") {
   tiledb::Config cfg;
-  cfg["sm.consolidation.total_buffer_size"] = "1048576";
+
+  cfg["sm.mem.consolidation.buffers_weight"] = "1";
+  cfg["sm.mem.consolidation.reader_weight"] = "5000";
+  cfg["sm.mem.consolidation.writer_weight"] = "5000";
   Context ctx(cfg);
   VFS vfs(ctx);
   const std::string array_name = "cpp_unit_array";
@@ -666,7 +680,7 @@ TEST_CASE(
   schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
 
-  tiledb::Array::create(array_name, schema);
+  tiledb::Array::create(ctx, array_name, schema);
   auto array_w = tiledb::Array(ctx, array_name, TILEDB_WRITE);
   std::vector<int> data = {0, 1};
 
@@ -702,7 +716,9 @@ TEST_CASE(
 TEST_CASE("C++ API: Encrypted array", "[cppapi][encryption][non-rest]") {
   const char key[] = "0123456789abcdeF0123456789abcdeF";
   tiledb::Config cfg;
-  cfg["sm.consolidation.total_buffer_size"] = "1048576";
+  cfg["sm.mem.consolidation.buffers_weight"] = "1";
+  cfg["sm.mem.consolidation.reader_weight"] = "5000";
+  cfg["sm.mem.consolidation.writer_weight"] = "5000";
   cfg["sm.encryption_type"] = "AES_256_GCM";
   cfg["sm.encryption_key"] = key;
   Context ctx(cfg);
@@ -720,7 +736,7 @@ TEST_CASE("C++ API: Encrypted array", "[cppapi][encryption][non-rest]") {
 
   REQUIRE_THROWS_AS(
       Array::encryption_type(ctx, array_name), tiledb::TileDBError);
-  Array::create(array_name, schema);
+  Array::create(ctx, array_name, schema);
   REQUIRE(Array::encryption_type(ctx, array_name) == TILEDB_AES_256_GCM);
 
   ArraySchema schema_read(ctx, array_name);
@@ -757,7 +773,9 @@ TEST_CASE("C++ API: Encrypted array", "[cppapi][encryption][non-rest]") {
   std::vector<int> subarray = {0, 3};
   std::vector<int> a_read(4);
   Query query_r(ctx, array);
-  query_r.set_subarray(subarray)
+  Subarray sub(ctx, array);
+  sub.set_subarray(subarray);
+  query_r.set_subarray(sub)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", a_read);
   query_r.submit();
@@ -769,7 +787,9 @@ TEST_CASE("C++ API: Encrypted array", "[cppapi][encryption][non-rest]") {
   Array array_3(ctx, array_name, TILEDB_READ);
   a_read = std::vector<int>(4, 0);
   Query query_r2(ctx, array_3);
-  query_r2.set_subarray(subarray)
+  Subarray sub2(ctx, array_3);
+  sub2.set_subarray(subarray);
+  query_r2.set_subarray(sub2)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", a_read);
   query_r2.submit();
@@ -789,7 +809,9 @@ TEST_CASE(
   tiledb::Config cfg;
   cfg["sm.encryption_type"] = "AES_256_GCM";
   cfg["sm.encryption_key"] = key.c_str();
-  cfg["sm.consolidation.total_buffer_size"] = "1048576";
+  cfg["sm.mem.consolidation.buffers_weight"] = "1";
+  cfg["sm.mem.consolidation.reader_weight"] = "5000";
+  cfg["sm.mem.consolidation.writer_weight"] = "5000";
   Context ctx(cfg);
   VFS vfs(ctx);
   const std::string array_name = "cpp_unit_array";
@@ -805,7 +827,7 @@ TEST_CASE(
 
   REQUIRE_THROWS_AS(
       Array::encryption_type(ctx, array_name), tiledb::TileDBError);
-  Array::create(array_name, schema);
+  Array::create(ctx, array_name, schema);
   REQUIRE(Array::encryption_type(ctx, array_name) == TILEDB_AES_256_GCM);
 
   ArraySchema schema_read(ctx, array_name);
@@ -843,7 +865,9 @@ TEST_CASE(
   std::vector<int> subarray = {0, 3};
   std::vector<int> a_read(4);
   Query query_r(ctx, array);
-  query_r.set_subarray(subarray)
+  Subarray sub(ctx, array);
+  sub.set_subarray(subarray);
+  query_r.set_subarray(sub)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", a_read);
   query_r.submit();
@@ -869,7 +893,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_DENSE);
   schema.set_domain(domain);
   schema.add_attribute(Attribute::create<int>(ctx, ""));
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   Array array(ctx, array_uri, TILEDB_READ);
   auto reloaded_schema = array.schema();
@@ -890,7 +914,7 @@ TEST_CASE("C++ API: Open array at", "[cppapi][open-array-at][rest]") {
   ArraySchema schema(ctx, TILEDB_DENSE);
   schema.set_domain(domain);
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   // Write array
   Array array_w(ctx, array_uri, TILEDB_WRITE);
@@ -909,7 +933,9 @@ TEST_CASE("C++ API: Open array at", "[cppapi][open-array-at][rest]") {
   std::vector<int> subarray = {1, 4};
   std::vector<int> a_r(4);
   Query query_r(ctx, array_r);
-  query_r.set_subarray(subarray)
+  Subarray sub(ctx, array_r);
+  sub.set_subarray(subarray);
+  query_r.set_subarray(sub)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", a_r);
   query_r.submit();
@@ -928,7 +954,9 @@ TEST_CASE("C++ API: Open array at", "[cppapi][open-array-at][rest]") {
 
   std::vector<int> a_r_at_0(4);
   Query query_r_at_0(ctx, array_r_at_0);
-  query_r_at_0.set_subarray(subarray)
+  Subarray sub2(ctx, array_r_at_0);
+  sub2.set_subarray(subarray);
+  query_r_at_0.set_subarray(sub2)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", a_r_at_0);
   query_r_at_0.submit();
@@ -950,7 +978,9 @@ TEST_CASE("C++ API: Open array at", "[cppapi][open-array-at][rest]") {
 
   std::vector<int> a_r_at(4);
   Query query_r_at(ctx, array_r_at);
-  query_r_at.set_subarray(subarray)
+  Subarray sub3(ctx, array_r_at);
+  sub3.set_subarray(subarray);
+  query_r_at.set_subarray(sub3)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", a_r_at_0);
   query_r_at.submit();
@@ -964,7 +994,9 @@ TEST_CASE("C++ API: Open array at", "[cppapi][open-array-at][rest]") {
   CHECK(array_reopen_at.open_timestamp_end() == first_timestamp);
   std::vector<int> a_r_reopen_at(4);
   Query query_r_reopen_at(ctx, array_reopen_at);
-  query_r_reopen_at.set_subarray(subarray)
+  Subarray sub4(ctx, array_reopen_at);
+  sub4.set_subarray(subarray);
+  query_r_reopen_at.set_subarray(sub4)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", a_r_reopen_at);
   query_r_reopen_at.submit();
@@ -991,7 +1023,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_DENSE);
   schema.set_domain(domain);
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
-  Array::create(array_name, schema);
+  Array::create(ctx, array_name, schema);
 
   // Write array
   Array array_w(ctx, array_name, TILEDB_WRITE);
@@ -1007,7 +1039,9 @@ TEST_CASE(
   std::vector<int> subarray = {1, 4};
   std::vector<int> a_r(4);
   Query query_r(ctx, array_r);
-  query_r.set_subarray(subarray)
+  Subarray sub(ctx, array_r);
+  sub.set_subarray(subarray);
+  query_r.set_subarray(sub)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", a_r);
   query_r.submit();
@@ -1031,7 +1065,9 @@ TEST_CASE(
 
   std::vector<int> a_r_at_0(4);
   Query query_r_at_0(ctx, array_r_at_0);
-  query_r_at_0.set_subarray(subarray)
+  Subarray sub2(ctx, array_r_at_0);
+  sub2.set_subarray(subarray);
+  query_r_at_0.set_subarray(sub2)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", a_r_at_0);
   query_r_at_0.submit();
@@ -1058,7 +1094,9 @@ TEST_CASE(
 
   std::vector<int> a_r_at(4);
   Query query_r_at(ctx, array_r_at);
-  query_r_at.set_subarray(subarray)
+  Subarray sub3(ctx, array_r_at);
+  sub3.set_subarray(subarray);
+  query_r_at.set_subarray(sub3)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", a_r_at_0);
   query_r_at.submit();
@@ -1083,7 +1121,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_SPARSE);
   schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   // Write
   std::vector<int> data_w = {1};
@@ -1105,7 +1143,9 @@ TEST_CASE(
   Query query(ctx, array);
   const std::vector<int> subarray = {0, 0, 0, 0};
   std::vector<int> data(1);
-  query.set_subarray(subarray)
+  Subarray sub(ctx, array);
+  sub.set_subarray(subarray);
+  query.set_subarray(sub)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", data);
   query.submit();
@@ -1132,7 +1172,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_DENSE);
   schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
   schema.add_attribute(Attribute::create(ctx, "a", datatype));
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   // Write
   std::byte data_w{1};
@@ -1173,6 +1213,7 @@ TEST_CASE(
                   .set_filter_list(FilterList(ctx).add_filter(
                       Filter(ctx, TILEDB_FILTER_BZIP2)));
   Array::create(
+      ctx,
       array_uri,
       ArraySchema(ctx, TILEDB_SPARSE)
           .set_domain(Domain(ctx).add_dimension(
@@ -1231,7 +1272,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_SPARSE);
   schema.add_attribute(a);
   schema.set_domain(dom);
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   // Write
   Array array(ctx, array_uri, TILEDB_WRITE);
@@ -1256,8 +1297,10 @@ TEST_CASE(
   query_r.set_data_buffer("d2", buff_d2_r);
   query_r.set_data_buffer("a", buff_a_r);
   query_r.set_layout(TILEDB_UNORDERED);
-  query_r.add_range(0, 1.0f, 20.0f);
-  query_r.add_range(1, (int64_t)1, (int64_t)30);
+  Subarray subarray_r(ctx, array_r);
+  subarray_r.add_range(0, 1.0f, 20.0f);
+  subarray_r.add_range(1, (int64_t)1, (int64_t)30);
+  query_r.set_subarray(subarray_r);
   query_r.submit();
 
   auto ret = query.result_buffer_elements();
@@ -1291,7 +1334,7 @@ TEST_CASE(
   auto a = Attribute::create<int32_t>(ctx, "a");
   schema.add_attribute(a);
   schema.set_domain(dom);
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   // Write
   Array array(ctx, array_uri, TILEDB_WRITE);
@@ -1324,48 +1367,53 @@ TEST_CASE(
   std::string s1("a", 1);
   std::string s2("ee", 2);
   Query query_r(ctx, array_r, TILEDB_READ);
+  Subarray subarray_r(ctx, array_r);
 
   SECTION("Non empty range") {
-    query_r.add_range(0, s1, s2);
-    CHECK_THROWS(query_r.add_range(1, s1, s2));
+    subarray_r.add_range(0, s1, s2);
+    CHECK_THROWS(subarray_r.add_range(1, s1, s2));
+    query_r.set_subarray(subarray_r);
 
     // Check range
-    CHECK_THROWS(query_r.range(1, 1));
-    std::array<std::string, 2> range = query_r.range(0, 0);
+    CHECK_THROWS(subarray_r.range(1, 1));
+    std::array<std::string, 2> range = subarray_r.range(0, 0);
     CHECK(range[0] == s1);
     CHECK(range[1] == s2);
   }
 
   SECTION("Empty first range") {
-    query_r.add_range(0, "", s2);
-    CHECK_THROWS(query_r.add_range(1, "", s2));
+    subarray_r.add_range(0, "", s2);
+    CHECK_THROWS(subarray_r.add_range(1, "", s2));
+    query_r.set_subarray(subarray_r);
 
     // Check range
-    CHECK_THROWS(query_r.range(1, 1));
-    std::array<std::string, 2> range = query_r.range(0, 0);
+    CHECK_THROWS(subarray_r.range(1, 1));
+    std::array<std::string, 2> range = subarray_r.range(0, 0);
     CHECK(range[0] == "");
     CHECK(range[1] == s2);
   }
 
   SECTION("Empty second range") {
-    query_r.add_range(0, s1, "");
-    CHECK_THROWS(query_r.add_range(1, s1, ""));
+    subarray_r.add_range(0, s1, "");
+    CHECK_THROWS(subarray_r.add_range(1, s1, ""));
+    query_r.set_subarray(subarray_r);
 
     // Check range
-    CHECK_THROWS(query_r.range(1, 1));
-    std::array<std::string, 2> range = query_r.range(0, 0);
+    CHECK_THROWS(subarray_r.range(1, 1));
+    std::array<std::string, 2> range = subarray_r.range(0, 0);
     CHECK(range[0] == s1);
     CHECK(range[1] == "");
     empty_results = true;
   }
 
   SECTION("Empty ranges") {
-    query_r.add_range(0, std::string(""), std::string(""));
-    CHECK_THROWS(query_r.add_range(1, std::string(""), std::string("")));
+    subarray_r.add_range(0, std::string(""), std::string(""));
+    CHECK_THROWS(subarray_r.add_range(1, std::string(""), std::string("")));
+    query_r.set_subarray(subarray_r);
 
     // Check range
-    CHECK_THROWS(query_r.range(1, 1));
-    std::array<std::string, 2> range = query_r.range(0, 0);
+    CHECK_THROWS(subarray_r.range(1, 1));
+    std::array<std::string, 2> range = subarray_r.range(0, 0);
     CHECK(range[0] == "");
     CHECK(range[1] == "");
     empty_results = true;
@@ -1418,7 +1466,7 @@ TEST_CASE(
   schema.set_tile_order(TILEDB_COL_MAJOR);
   schema.set_cell_order(TILEDB_COL_MAJOR);
   schema.set_domain(dom);
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   // Write
   Array array(ctx, array_uri, TILEDB_WRITE);
@@ -1448,7 +1496,9 @@ TEST_CASE(
   std::string s1("a", 1);
   std::string s2("ee", 2);
   Query query_r(ctx, array_r, TILEDB_READ);
-  query_r.add_range(0, s1, s2);
+  Subarray subarray_r(ctx, array_r);
+  subarray_r.add_range(0, s1, s2);
+  query_r.set_subarray(subarray_r);
   std::string data;
   data.resize(10);
   std::vector<uint64_t> offsets(4);
@@ -1477,7 +1527,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_SPARSE);
   schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   // Write
   std::vector<int> data_w = {1};
@@ -1498,7 +1548,9 @@ TEST_CASE(
   Query query(ctx, array);
   const std::vector<int> subarray = {0, 0, 0, 0};
   std::vector<int> rows(1);
-  query.set_subarray(subarray)
+  Subarray sub(ctx, array);
+  sub.set_subarray(subarray);
+  query.set_subarray(sub)
       .set_layout(TILEDB_GLOBAL_ORDER)
       .set_data_buffer("rows", rows);
   query.submit();
@@ -1522,7 +1574,7 @@ TEST_CASE(
   schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
   schema.set_allows_dups(true);
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   // Write
   std::vector<int> data_w = {1};
@@ -1543,7 +1595,9 @@ TEST_CASE(
   Query query(ctx, array);
   const std::vector<int> subarray = {0, 0, 0, 0};
   std::vector<int> rows(1);
-  query.set_subarray(subarray)
+  Subarray sub(ctx, array);
+  sub.set_subarray(subarray);
+  query.set_subarray(sub)
       .set_layout(TILEDB_UNORDERED)
       .set_data_buffer("rows", rows);
   query.submit();
@@ -1566,7 +1620,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_DENSE);
   schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   // Write
   std::vector<int> data_w = {
@@ -1582,10 +1636,10 @@ TEST_CASE(
   // Read
   Array array(ctx, array_uri, TILEDB_READ);
   Query query(ctx, array);
+  Subarray subarray(ctx, array);
+  subarray.add_range(0, 0, 1).add_range(0, 3, 3).add_range(1, 0, 3);
   std::vector<int> data(12);
-  query.add_range(0, 0, 1)
-      .add_range(0, 3, 3)
-      .add_range(1, 0, 3)
+  query.set_subarray(subarray)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", data);
   query.submit();
@@ -1614,7 +1668,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_DENSE);
   schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
-  Array::create(array_uri, schema);
+  Array::create(ctx, array_uri, schema);
 
   // Write
   std::vector<int> data_w = {
@@ -1644,14 +1698,10 @@ TEST_CASE(
   // cpp_unit_array/__schema
   // cpp_unit_array/__meta
   // cpp_unit_array/__fragment_meta
-  CHECK(
-      stats.find("\"Context.StorageManager.VFS.read_ops_num\": 5") !=
-      std::string::npos);
+  CHECK(stats.find("\"Context.VFS.read_ops_num\": 5") != std::string::npos);
 
   // Expect file_size on the fragment.
-  CHECK(
-      stats.find("\"Context.StorageManager.VFS.file_size_num\": 1") !=
-      std::string::npos);
+  CHECK(stats.find("\"Context.VFS.file_size_num\": 1") != std::string::npos);
 }
 
 TEST_CASE(
@@ -1671,7 +1721,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_SPARSE);
   schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
-  Array::create(array_name, schema);
+  Array::create(ctx, array_name, schema);
 
   // Write
   std::vector<int> data_w = {1};
@@ -1704,14 +1754,10 @@ TEST_CASE(
   // cpp_unit_array/__schema
   // cpp_unit_array/__meta
   // cpp_unit_array/__fragment_meta
-  CHECK(
-      stats.find("\"Context.StorageManager.VFS.read_ops_num\": 5") !=
-      std::string::npos);
+  CHECK(stats.find("\"Context.VFS.read_ops_num\": 5") != std::string::npos);
 
   // Expect file_size on the fragment.
-  CHECK(
-      stats.find("\"Context.StorageManager.VFS.file_size_num\": 1") !=
-      std::string::npos);
+  CHECK(stats.find("\"Context.VFS.file_size_num\": 1") != std::string::npos);
 }
 
 TEST_CASE(
@@ -1726,7 +1772,7 @@ TEST_CASE(
   ArraySchema schema(ctx, TILEDB_DENSE);
   schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
   schema.add_attribute(Attribute::create<int>(ctx, "a"));
-  Array::create(array_name, schema);
+  Array::create(ctx, array_name, schema);
 
   // Try writing on a non-process-global Context
   Context ctx_non_global;
@@ -1747,9 +1793,10 @@ TEST_CASE(
   // Read
   Array array(ctx, array_name, TILEDB_READ);
   Query query(ctx, array, TILEDB_READ);
+  Subarray subarray(ctx, array);
+  subarray.add_range(0, 1, 2).add_range(1, 2, 4);
   std::vector<int> data(6);
-  query.add_range(0, 1, 2)
-      .add_range(1, 2, 4)
+  query.set_subarray(subarray)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("a", data);
   query.submit();
@@ -1798,7 +1845,7 @@ TEST_CASE(
   schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
   Attribute attr = Attribute::create<int>(ctx, "a");
   schema.add_attribute(attr);
-  Array::create(array_name, schema);
+  Array::create(ctx, array_name, schema);
   REQUIRE(vfs.ls(array_name).size() == 1);
 
   // Ensure the array can be opened and write to it
@@ -1929,7 +1976,8 @@ TEST_CASE(
 
   // Read from upgraded version
   Array array_read(ctx, old_array_name, TILEDB_READ);
-  std::vector<int> subarray_read = {1, 4, 10, 10};
+  Subarray subarray_read(ctx, array_read);
+  subarray_read.set_subarray({1, 4, 10, 10});
   std::vector<int> a_read;
   a_read.resize(4);
   std::vector<int> d1_read;
@@ -1975,54 +2023,6 @@ TEST_CASE(
   vfs.remove_dir(array_read.uri() + "/__schema");
 }
 
-TEST_CASE(
-    "C++ API: Close array with running query",
-    "[cppapi][close-before-read][non-rest]") {
-  // async queries not supported on remote arrays (REST)
-  tiledb::test::VFSTestSetup vfs_test_setup{};
-  Context ctx{vfs_test_setup.ctx()};
-  auto array_uri{vfs_test_setup.array_uri("cpp_unit_array")};
-
-  // Create
-  Domain domain(ctx);
-  domain.add_dimension(Dimension::create<int>(ctx, "rows", {{0, 3}}, 4))
-      .add_dimension(Dimension::create<int>(ctx, "cols", {{0, 3}}, 4));
-  ArraySchema schema(ctx, TILEDB_DENSE);
-  schema.set_domain(domain).set_order({{TILEDB_ROW_MAJOR, TILEDB_ROW_MAJOR}});
-  schema.add_attribute(Attribute::create<int>(ctx, "a"));
-  Array::create(array_uri, schema);
-
-  // Write
-  std::vector<int> data_w = {
-      1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
-  Array array_w(ctx, array_uri, TILEDB_WRITE);
-  Query query_w(ctx, array_w);
-  query_w.set_subarray(Subarray(ctx, array_w).set_subarray({0, 3, 0, 3}))
-      .set_layout(TILEDB_ROW_MAJOR)
-      .set_data_buffer("a", data_w);
-  query_w.submit();
-  array_w.close();
-
-  // Open for read.
-  Array array(ctx, array_uri, TILEDB_READ);
-  std::vector<int> subarray_read = {0, 3, 0, 3};
-  std::vector<int> a_read(16);
-
-  Query query(ctx, array);
-  query.set_subarray(subarray_read)
-      .set_layout(TILEDB_ROW_MAJOR)
-      .set_data_buffer("a", a_read);
-  query.submit_async();
-  array.close();
-
-  uint64_t i = 0;
-  while (query.query_status() != Query::Status::COMPLETE) {
-    i++;
-  }
-
-  CHECK(i > 0);
-}
-
 TEST_CASE("C++ API: Read empty array", "[cppapi][read-empty-array]") {
   const std::string array_name_1d = "cpp_unit_array_1d";
   Context ctx;
@@ -2040,7 +2040,7 @@ TEST_CASE("C++ API: Read empty array", "[cppapi][read-empty-array]") {
   schema.set_domain(domain);
   schema.add_attribute(Attribute::create<int32_t>(ctx, "a"));
   schema.set_allows_dups(dups);
-  Array::create(array_name_1d, schema);
+  Array::create(ctx, array_name_1d, schema);
   Array array(ctx, array_name_1d, TILEDB_READ);
 
   std::vector<int32_t> d(1);
