@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2020-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2020-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -34,7 +34,6 @@
 #include "test/support/src/helpers.h"
 #include "test/support/src/vfs_helpers.h"
 #include "tiledb/sm/cpp_api/tiledb"
-#include "tiledb/sm/misc/utils.h"
 
 #include <iostream>
 #include <vector>
@@ -314,7 +313,9 @@ void NullableArrayCppFx::read(
   }
 
   // Set the subarray to read.
-  query.set_subarray(subarray);
+  Subarray sub(ctx_, array);
+  sub.set_subarray(subarray);
+  query.set_subarray(sub);
 
   // Submit the query.
   REQUIRE(query.submit() == Query::Status::COMPLETE);
