@@ -167,12 +167,14 @@ TEST_CASE(
   // Prepare a read query.
   Array array_read(ctx, array_name, TILEDB_READ);
   Query query_read(ctx, array_read, TILEDB_READ);
+  Subarray subarray_read(ctx, array_read);
   auto dim1_non_empty_domain = array_read.non_empty_domain_var(0);
   auto dim2_non_empty_domain = array_read.non_empty_domain<int32_t>(1);
-  query_read.add_range(
+  subarray_read.add_range(
       0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-  query_read.add_range<int32_t>(
+  subarray_read.add_range<int32_t>(
       1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
+  query_read.set_subarray(subarray_read);
 
   // Prepare buffers with small enough sizes to ensure the string dimension
   // must split.
@@ -254,6 +256,7 @@ TEST_CASE(
     // Prepare a read query.
     Array array_read(ctx, array_name, TILEDB_READ);
     Query query_read(ctx, array_read, TILEDB_READ);
+    Subarray subarray_read(ctx, array_read);
     auto dim1_non_empty_domain = array_read.non_empty_domain_var(0);
     auto dim2_non_empty_domain = array_read.non_empty_domain<int32_t>(1);
     auto dim3_non_empty_domain = array_read.non_empty_domain_var(2);
@@ -264,7 +267,7 @@ TEST_CASE(
     // editors.
     switch (option) {
       case 1:
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
         expected_dim1 = {"bb", "c", "d"};
         expected_dim2 = {1, 1, 2};
@@ -272,11 +275,11 @@ TEST_CASE(
         expected_a1_data = {2, 3, 4};
         break;
       case 2:
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"c", "d"};
         expected_dim2 = {1, 2};
@@ -284,9 +287,9 @@ TEST_CASE(
         expected_a1_data = {3, 4};
         break;
       case 3:
-        query_read.add_range(
+        subarray_read.add_range(
             0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"a", "bb", "c", "d", "ee", "f"};
         expected_dim2 = {1, 1, 1, 2, 2, 2};
@@ -294,15 +297,15 @@ TEST_CASE(
         expected_a1_data = {1, 2, 3, 4, 5, 6};
         break;
       case 4:
-        query_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
         expected_dim1 = {"c", "d"};
         expected_dim2 = {1, 2};
         expected_dim3 = {"i", "j"};
         expected_a1_data = {3, 4};
         break;
       case 5:
-        query_read.add_range(0, std::string("c"), std::string("d"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"c", "d"};
         expected_dim2 = {1, 2};
@@ -310,9 +313,9 @@ TEST_CASE(
         expected_a1_data = {3, 4};
         break;
       case 6:
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"bb", "c", "d"};
         expected_dim2 = {1, 1, 2};
@@ -320,9 +323,9 @@ TEST_CASE(
         expected_a1_data = {2, 3, 4};
         break;
       case 7:
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"c", "d"};
         expected_dim2 = {1, 2};
@@ -330,8 +333,8 @@ TEST_CASE(
         expected_a1_data = {3, 4};
         break;
       case 8:
-        query_read.add_range(0, std::string("c"), std::string("d"));
-        query_read.add_range(
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"c", "d"};
         expected_dim2 = {1, 2};
@@ -339,9 +342,9 @@ TEST_CASE(
         expected_a1_data = {3, 4};
         break;
       case 9:
-        query_read.add_range(
+        subarray_read.add_range(
             0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"c", "d", "ee"};
         expected_dim2 = {1, 2, 2};
@@ -349,8 +352,8 @@ TEST_CASE(
         expected_a1_data = {3, 4, 5};
         break;
       case 10:
-        query_read.add_range(0, std::string("c"), std::string("ee"));
-        query_read.add_range(
+        subarray_read.add_range(0, std::string("c"), std::string("ee"));
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kk"));
         expected_dim1 = {"c", "d", "ee"};
         expected_dim2 = {1, 2, 2};
@@ -358,6 +361,7 @@ TEST_CASE(
         expected_a1_data = {3, 4, 5};
         break;
     }
+    query_read.set_subarray(subarray_read);
 
     std::vector<char> dim1;
     std::vector<uint64_t> dim1_offsets;
@@ -481,6 +485,7 @@ TEST_CASE(
     // Prepare a read query. Do not set a range for the last string dimension.
     Array array_read(ctx, array_name, TILEDB_READ);
     Query query_read(ctx, array_read, TILEDB_READ);
+    Subarray subarray_read(ctx, array_read);
     auto dim1_non_empty_domain = array_read.non_empty_domain_var(0);
     auto dim2_non_empty_domain = array_read.non_empty_domain<int32_t>(1);
 
@@ -497,7 +502,7 @@ TEST_CASE(
       case 1:
         expected_result_num = 4;
         initial_expected_read_status = Query::Status::INCOMPLETE;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
         expected_dim1 = {"bb", "bb", "c", "c"};
         expected_dim2 = {1, 2, 1, 2};
@@ -507,11 +512,11 @@ TEST_CASE(
       case 2:
         expected_result_num = 2;
         initial_expected_read_status = Query::Status::COMPLETE;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"bb", "c"};
         expected_dim2 = {2, 1};
@@ -521,9 +526,9 @@ TEST_CASE(
       case 3:
         expected_result_num = 6;
         initial_expected_read_status = Query::Status::INCOMPLETE;
-        query_read.add_range(
+        subarray_read.add_range(
             0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"a", "a", "bb", "bb", "c", "c"};
         expected_dim2 = {1, 2, 1, 2, 1, 2};
@@ -533,7 +538,7 @@ TEST_CASE(
       case 4:
         expected_result_num = 2;
         initial_expected_read_status = Query::Status::COMPLETE;
-        query_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
         expected_dim1 = {"c", "c"};
         expected_dim2 = {1, 2};
         expected_dim3 = {"i", "l"};
@@ -542,8 +547,8 @@ TEST_CASE(
       case 5:
         expected_result_num = 2;
         initial_expected_read_status = Query::Status::COMPLETE;
-        query_read.add_range(0, std::string("c"), std::string("d"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"c", "c"};
         expected_dim2 = {1, 2};
@@ -553,9 +558,9 @@ TEST_CASE(
       case 6:
         expected_result_num = 4;
         initial_expected_read_status = Query::Status::INCOMPLETE;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"bb", "bb", "c", "c"};
         expected_dim2 = {1, 2, 1, 2};
@@ -565,9 +570,9 @@ TEST_CASE(
       case 7:
         expected_result_num = 2;
         initial_expected_read_status = Query::Status::COMPLETE;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"bb", "c"};
         expected_dim2 = {2, 1};
@@ -578,8 +583,8 @@ TEST_CASE(
         expected_result_num = 1;
         initial_result_num = 1;
         initial_expected_read_status = Query::Status::COMPLETE;
-        query_read.add_range(0, std::string("c"), std::string("d"));
-        query_read.add_range(
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"c"};
         expected_dim2 = {1};
@@ -590,9 +595,9 @@ TEST_CASE(
         expected_result_num = 3;
         initial_result_num = 3;
         initial_expected_read_status = Query::Status::COMPLETE;
-        query_read.add_range(
+        subarray_read.add_range(
             0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"a", "bb", "c"};
         expected_dim2 = {2, 2, 1};
@@ -600,6 +605,7 @@ TEST_CASE(
         expected_a1_data = {4, 5, 3};
         break;
     }
+    query_read.set_subarray(subarray_read);
 
     std::vector<char> dim1;
     std::vector<uint64_t> dim1_offsets;
@@ -721,6 +727,7 @@ TEST_CASE(
   // Prepare a read query. Do not set a range for the last string dimension.
   Array array_read(ctx, array_name, TILEDB_READ);
   Query query_read(ctx, array_read, TILEDB_READ);
+  Subarray subarray_read(ctx, array_read);
   auto dim1_non_empty_domain = array_read.non_empty_domain_var(0);
   auto dim2_non_empty_domain = array_read.non_empty_domain<int32_t>(1);
 
@@ -738,7 +745,7 @@ TEST_CASE(
     which_option = 1;
     expected_result_num = 4;
     initial_expected_read_status = Query::Status::INCOMPLETE;
-    query_read.add_range(
+    subarray_read.add_range(
         std::string("dim1"), std::string("az"), std::string("de"));
     expected_dim1 = {"bb", "bb", "c", "c"};
     expected_dim2 = {1, 2, 1, 2};
@@ -749,11 +756,11 @@ TEST_CASE(
     which_option = 2;
     expected_result_num = 2;
     initial_expected_read_status = Query::Status::COMPLETE;
-    query_read.add_range(
+    subarray_read.add_range(
         std::string("dim1"), std::string("az"), std::string("de"));
-    query_read.add_range<int32_t>(
+    subarray_read.add_range<int32_t>(
         1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
-    query_read.add_range(
+    subarray_read.add_range(
         std::string("dim3"), std::string("i"), std::string("kl"));
     expected_dim1 = {"bb", "c"};
     expected_dim2 = {2, 1};
@@ -764,9 +771,9 @@ TEST_CASE(
     which_option = 3;
     expected_result_num = 6;
     initial_expected_read_status = Query::Status::INCOMPLETE;
-    query_read.add_range(
+    subarray_read.add_range(
         0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-    query_read.add_range<int32_t>(
+    subarray_read.add_range<int32_t>(
         1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
     expected_dim1 = {"a", "a", "bb", "bb", "c", "c"};
     expected_dim2 = {1, 2, 1, 2, 1, 2};
@@ -777,7 +784,7 @@ TEST_CASE(
     which_option = 4;
     expected_result_num = 2;
     initial_expected_read_status = Query::Status::COMPLETE;
-    query_read.add_range(0, std::string("c"), std::string("d"));
+    subarray_read.add_range(0, std::string("c"), std::string("d"));
     expected_dim1 = {"c", "c"};
     expected_dim2 = {1, 2};
     expected_dim3 = {"i", "l"};
@@ -787,8 +794,8 @@ TEST_CASE(
     which_option = 5;
     expected_result_num = 2;
     initial_expected_read_status = Query::Status::COMPLETE;
-    query_read.add_range(0, std::string("c"), std::string("d"));
-    query_read.add_range<int32_t>(
+    subarray_read.add_range(0, std::string("c"), std::string("d"));
+    subarray_read.add_range<int32_t>(
         1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
     expected_dim1 = {"c", "c"};
     expected_dim2 = {1, 2};
@@ -799,9 +806,9 @@ TEST_CASE(
     which_option = 6;
     expected_result_num = 4;
     initial_expected_read_status = Query::Status::INCOMPLETE;
-    query_read.add_range(
+    subarray_read.add_range(
         std::string("dim1"), std::string("az"), std::string("de"));
-    query_read.add_range<int32_t>(
+    subarray_read.add_range<int32_t>(
         1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
     expected_dim1 = {"bb", "bb", "c", "c"};
     expected_dim2 = {1, 2, 1, 2};
@@ -812,9 +819,9 @@ TEST_CASE(
     which_option = 7;
     expected_result_num = 2;
     initial_expected_read_status = Query::Status::COMPLETE;
-    query_read.add_range(
+    subarray_read.add_range(
         std::string("dim1"), std::string("az"), std::string("de"));
-    query_read.add_range(
+    subarray_read.add_range(
         std::string("dim3"), std::string("i"), std::string("kl"));
     expected_dim1 = {"bb", "c"};
     expected_dim2 = {2, 1};
@@ -826,8 +833,8 @@ TEST_CASE(
     expected_result_num = 1;
     initial_result_num = 1;
     initial_expected_read_status = Query::Status::COMPLETE;
-    query_read.add_range(0, std::string("c"), std::string("d"));
-    query_read.add_range(
+    subarray_read.add_range(0, std::string("c"), std::string("d"));
+    subarray_read.add_range(
         std::string("dim3"), std::string("i"), std::string("kl"));
     expected_dim1 = {"c"};
     expected_dim2 = {1};
@@ -839,15 +846,16 @@ TEST_CASE(
     expected_result_num = 3;
     initial_result_num = 3;
     initial_expected_read_status = Query::Status::COMPLETE;
-    query_read.add_range(
+    subarray_read.add_range(
         0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-    query_read.add_range(
+    subarray_read.add_range(
         std::string("dim3"), std::string("i"), std::string("kl"));
     expected_dim1 = {"a", "bb", "c"};
     expected_dim2 = {2, 2, 1};
     expected_dim3 = {"j", "kk", "i"};
     expected_a1_data = {4, 5, 3};
   }
+  query_read.set_subarray(subarray_read);
 
   std::vector<uint64_t> dim1_offsets;
   std::vector<int32_t> dim2;
@@ -995,6 +1003,7 @@ TEST_CASE(
     // Prepare a read query. Ranges set vary in the different switch cases.
     Array array_read(ctx, array_name, TILEDB_READ);
     Query query_read(ctx, array_read, TILEDB_READ);
+    Subarray subarray_read(ctx, array_read);
     auto dim1_non_empty_domain = array_read.non_empty_domain_var(0);
     auto dim2_non_empty_domain = array_read.non_empty_domain<int32_t>(1);
 
@@ -1007,7 +1016,7 @@ TEST_CASE(
     switch (option) {
       case 1:
         expected_result_num = 4;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
         expected_dim1 = {"bb", "bb", "c", "c"};
         expected_dim2 = {1, 2, 1, 2};
@@ -1017,11 +1026,11 @@ TEST_CASE(
         break;
       case 2:
         expected_result_num = 2;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"bb", "c"};
         expected_dim2 = {2, 1};
@@ -1030,9 +1039,9 @@ TEST_CASE(
         break;
       case 3:
         expected_result_num = 6;
-        query_read.add_range(
+        subarray_read.add_range(
             0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"a", "a", "bb", "bb", "c", "c"};
         expected_dim2 = {1, 2, 1, 2, 1, 2};
@@ -1041,7 +1050,7 @@ TEST_CASE(
         break;
       case 4:
         expected_result_num = 2;
-        query_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
         expected_dim1 = {"c", "c"};
         expected_dim2 = {1, 2};
         expected_dim3 = {"i", "l"};
@@ -1049,8 +1058,8 @@ TEST_CASE(
         break;
       case 5:
         expected_result_num = 2;
-        query_read.add_range(0, std::string("c"), std::string("d"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"c", "c"};
         expected_dim2 = {1, 2};
@@ -1059,9 +1068,9 @@ TEST_CASE(
         break;
       case 6:
         expected_result_num = 4;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"bb", "bb", "c", "c"};
         expected_dim2 = {1, 2, 1, 2};
@@ -1070,9 +1079,9 @@ TEST_CASE(
         break;
       case 7:
         expected_result_num = 2;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"bb", "c"};
         expected_dim2 = {2, 1};
@@ -1081,8 +1090,8 @@ TEST_CASE(
         break;
       case 8:
         expected_result_num = 1;
-        query_read.add_range(0, std::string("c"), std::string("d"));
-        query_read.add_range(
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"c"};
         expected_dim2 = {1};
@@ -1091,9 +1100,9 @@ TEST_CASE(
         break;
       case 9:
         expected_result_num = 3;
-        query_read.add_range(
+        subarray_read.add_range(
             0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"a", "bb", "c"};
         expected_dim2 = {2, 2, 1};
@@ -1101,6 +1110,7 @@ TEST_CASE(
         expected_a1_data = {4, 5, 3};
         break;
     }
+    query_read.set_subarray(subarray_read);
 
     std::vector<char> dim1;
     std::vector<uint64_t> dim1_offsets;
@@ -1247,6 +1257,7 @@ TEST_CASE(
     // Prepare a read query. Ranges set vary in the different switch cases.
     Array array_read(ctx, array_name, TILEDB_READ);
     Query query_read(ctx, array_read, TILEDB_READ);
+    Subarray subarray_read(ctx, array_read);
     auto dim1_non_empty_domain = array_read.non_empty_domain_var(0);
     auto dim2_non_empty_domain = array_read.non_empty_domain<int32_t>(1);
 
@@ -1259,7 +1270,7 @@ TEST_CASE(
     switch (option) {
       case 1:
         expected_result_num = 4;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
         expected_dim1 = {"bb", "bb", "c", "c"};
         expected_dim2 = {1, 2, 1, 2};
@@ -1269,11 +1280,11 @@ TEST_CASE(
         break;
       case 2:
         expected_result_num = 2;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"bb", "c"};
         expected_dim2 = {2, 1};
@@ -1282,9 +1293,9 @@ TEST_CASE(
         break;
       case 3:
         expected_result_num = 6;
-        query_read.add_range(
+        subarray_read.add_range(
             0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"a", "a", "bb", "bb", "c", "c"};
         expected_dim2 = {1, 2, 1, 2, 1, 2};
@@ -1293,7 +1304,7 @@ TEST_CASE(
         break;
       case 4:
         expected_result_num = 2;
-        query_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
         expected_dim1 = {"c", "c"};
         expected_dim2 = {1, 2};
         expected_dim3 = {"i", "l"};
@@ -1301,8 +1312,8 @@ TEST_CASE(
         break;
       case 5:
         expected_result_num = 2;
-        query_read.add_range(0, std::string("c"), std::string("d"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"c", "c"};
         expected_dim2 = {1, 2};
@@ -1311,9 +1322,9 @@ TEST_CASE(
         break;
       case 6:
         expected_result_num = 4;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range<int32_t>(
+        subarray_read.add_range<int32_t>(
             1, dim2_non_empty_domain.first, dim2_non_empty_domain.second);
         expected_dim1 = {"bb", "bb", "c", "c"};
         expected_dim2 = {1, 2, 1, 2};
@@ -1322,9 +1333,9 @@ TEST_CASE(
         break;
       case 7:
         expected_result_num = 2;
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim1"), std::string("az"), std::string("de"));
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"bb", "c"};
         expected_dim2 = {2, 1};
@@ -1333,8 +1344,8 @@ TEST_CASE(
         break;
       case 8:
         expected_result_num = 1;
-        query_read.add_range(0, std::string("c"), std::string("d"));
-        query_read.add_range(
+        subarray_read.add_range(0, std::string("c"), std::string("d"));
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"c"};
         expected_dim2 = {1};
@@ -1343,9 +1354,9 @@ TEST_CASE(
         break;
       case 9:
         expected_result_num = 3;
-        query_read.add_range(
+        subarray_read.add_range(
             0, dim1_non_empty_domain.first, dim1_non_empty_domain.second);
-        query_read.add_range(
+        subarray_read.add_range(
             std::string("dim3"), std::string("i"), std::string("kl"));
         expected_dim1 = {"a", "bb", "c"};
         expected_dim2 = {2, 2, 1};
@@ -1353,6 +1364,7 @@ TEST_CASE(
         expected_a1_data = {4, 5, 3};
         break;
     }
+    query_read.set_subarray(subarray_read);
 
     std::vector<char> dim1;
     std::vector<uint64_t> dim1_offsets;
@@ -1479,8 +1491,12 @@ void read_and_check_sparse_array_string_dim(
   std::string data_back;
   data_back.resize(expected_data.size());
 
+  Subarray subarray(ctx, array);
+  subarray.add_range(
+      "dim1", std::string("ATSD987JIO"), std::string("TGSD987JPO"));
+
   Query query(ctx, array, TILEDB_READ);
-  query.add_range("dim1", std::string("ATSD987JIO"), std::string("TGSD987JPO"));
+  query.set_subarray(subarray);
   query.set_data_buffer("dim1", (char*)data_back.data(), data_back.size());
   query.set_offsets_buffer("dim1", offsets_back.data(), offsets_back.size());
   query.set_layout(layout);
