@@ -44,6 +44,7 @@
 #endif
 
 #include <test/support/tdb_catch.h>
+#include <numeric>
 
 using namespace tiledb;
 using namespace tiledb::test;
@@ -365,8 +366,14 @@ int32_t CSparseGlobalOrderFx::read(
 
   if (set_subarray) {
     // Set subarray.
-    rc = tiledb_query_set_subarray(ctx_, query, subarray.data());
+    tiledb_subarray_t* sub;
+    rc = tiledb_subarray_alloc(ctx_, array, &sub);
     CHECK(rc == TILEDB_OK);
+    rc = tiledb_subarray_set_subarray(ctx_, sub, subarray.data());
+    CHECK(rc == TILEDB_OK);
+    rc = tiledb_query_set_subarray_t(ctx_, query, sub);
+    CHECK(rc == TILEDB_OK);
+    tiledb_subarray_free(&sub);
   }
 
   if (qc_idx != 0) {
@@ -448,8 +455,14 @@ int32_t CSparseGlobalOrderFx::read_strings(
 
   if (set_subarray) {
     // Set subarray.
-    rc = tiledb_query_set_subarray(ctx_, query, subarray.data());
+    tiledb_subarray_t* sub;
+    rc = tiledb_subarray_alloc(ctx_, array, &sub);
     CHECK(rc == TILEDB_OK);
+    rc = tiledb_subarray_set_subarray(ctx_, sub, subarray.data());
+    CHECK(rc == TILEDB_OK);
+    rc = tiledb_query_set_subarray_t(ctx_, query, sub);
+    CHECK(rc == TILEDB_OK);
+    tiledb_subarray_free(&sub);
   }
 
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
@@ -776,10 +789,10 @@ TEST_CASE_METHOD(
     write_1d_fragment(coords, &coords_size, data, &data_size);
   }
 
-  // Two result tile (2 * (~1200 + 8) will be bigger than the per fragment
+  // Two result tile (2 * (~3000 + 8) will be bigger than the per fragment
   // budget (1000).
-  total_budget_ = "12000";
-  ratio_coords_ = "0.30";
+  total_budget_ = "35000";
+  ratio_coords_ = "0.11";
   update_config();
 
   tiledb_array_t* array = nullptr;
@@ -810,8 +823,7 @@ TEST_CASE_METHOD(
   REQUIRE(stats != nullptr);
   auto counters = stats->counters();
   REQUIRE(counters != nullptr);
-  auto loop_num =
-      counters->find("Context.StorageManager.Query.Reader.internal_loop_num");
+  auto loop_num = counters->find("Context.Query.Reader.internal_loop_num");
   CHECK(5 == loop_num->second);
 
   // Check incomplete query status.
@@ -858,7 +870,7 @@ TEST_CASE_METHOD(
   write_1d_fragment(coords, &coords_size, data, &data_size);
 
   // One result tile (8 + ~440) will be bigger than the budget (400).
-  total_budget_ = "10000";
+  total_budget_ = "19000";
   ratio_coords_ = "0.04";
   update_config();
 
@@ -1195,7 +1207,7 @@ TEST_CASE(
     "Sparse global order reader: attribute copy memory limit",
     "[sparse-global-order][attribute-copy][memory-limit][rest]") {
   Config config;
-  config["sm.mem.total_budget"] = "10000";
+  config["sm.mem.total_budget"] = "20000";
   VFSTestSetup vfs_test_setup(config.ptr().get());
   std::string array_name = vfs_test_setup.array_uri("test_sparse_global_order");
   auto ctx = vfs_test_setup.ctx();
@@ -1300,10 +1312,10 @@ TEST_CASE_METHOD(
     write_1d_fragment(coords, &coords_size, data, &data_size);
   }
 
-  // Two result tile (2 * (~1200 + 8) will be bigger than the per fragment
+  // Two result tile (2 * (~4000 + 8) will be bigger than the per fragment
   // budget (1000).
-  total_budget_ = "12000";
-  ratio_coords_ = "0.30";
+  total_budget_ = "40000";
+  ratio_coords_ = "0.22";
   update_config();
 
   tiledb_array_t* array = nullptr;
