@@ -36,6 +36,7 @@
 #include "test/support/src/coords_workaround.h"
 #include "test/support/src/mem_helpers.h"
 #include "tiledb.h"
+#include "tiledb/api/c_api/subarray/subarray_api_internal.h"
 #include "tiledb/common/common.h"
 #include "tiledb/common/random/random_label.h"
 #include "tiledb/sm/array/array.h"
@@ -43,7 +44,6 @@
 #include "tiledb/sm/enums/layout.h"
 #include "tiledb/sm/enums/serialization_type.h"
 #include "tiledb/sm/stats/stats.h"
-#include "tiledb/sm/subarray/subarray.h"
 #include "tiledb_serialization.h"
 
 #include <mutex>
@@ -917,6 +917,56 @@ int deserialize_array_and_query(
     const char* array_uri,
     bool clientside);
 
+/**
+ * Helper function that generates a fragment uri for an array.
+ * If the `array` arg is specified, the full URI is generated using
+ * the write version of the array schema and the timestamp
+ * the array was opened at. If unspecified, the function
+ * will generate only the fragment name using the current timestamp
+ * and the latest library format version.
+ *
+ * @param array A TileDB array
+ * @return A test fragment uri
+ */
+sm::URI generate_fragment_uri(sm::Array* array);
+
+/**
+ * Helper function to create a sparse array using format version 11.
+ *
+ * @param ctx TileDB context.
+ * @param array_name The name of the new array to create.
+ */
+void create_sparse_array_v11(tiledb_ctx_t* ctx, const std::string& array_name);
+
+/**
+ * Helper function to write data to a format version 11 sparse array.
+ *
+ * @param ctx TileDB context.
+ * @param array_name The name of the array to write to.
+ * @param timestamp The timestamp to open the array for writing.
+ */
+void write_sparse_v11(
+    tiledb_ctx_t* ctx, const std::string& array_name, uint64_t timestamp);
+
+/**
+ * Helper function to validate data read from a format version 11 sparse array.
+ *
+ * @param ctx TileDB context.
+ * @param array_name The name of the array to read from.
+ * @param timestamp The timestamp to open the array for reading.
+ */
+void read_sparse_v11(
+    tiledb_ctx_t* ctx, const std::string& array_name, uint64_t timestamp);
+
+/**
+ * Helper function to test two array schemas are equivalent.
+ *
+ * @param schema1 Expected array schema.
+ * @param schema2 Actual array schema.
+ */
+void schema_equiv(
+    const sm::ArraySchema& schema1, const sm::ArraySchema& schema2);
+
 }  // namespace tiledb::test
 
 #endif
