@@ -292,7 +292,10 @@ TEST_CASE_METHOD(QueryFieldFx, "C API: get_field", "[capi][query_field]") {
 
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
   int64_t dom[] = {1, 9, 1, 2};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(tiledb_subarray_set_subarray(ctx, subarray, &dom) == TILEDB_OK);
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   tiledb_query_field_t* field = nullptr;
   tiledb_datatype_t type;
@@ -393,4 +396,5 @@ TEST_CASE_METHOD(QueryFieldFx, "C API: get_field", "[capi][query_field]") {
   tiledb_query_free(&query);
   CHECK(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
+  tiledb_subarray_free(&subarray);
 }
