@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -36,7 +36,7 @@
 #ifndef TILEDB_LOGGER_H
 #define TILEDB_LOGGER_H
 
-#include <fmt/format.h>
+#include <spdlog/fmt/fmt.h>
 #include <atomic>
 #include <sstream>
 
@@ -48,8 +48,7 @@ namespace spdlog {
 class logger;
 }
 
-namespace tiledb {
-namespace common {
+namespace tiledb::common {
 
 /** Definition of class Logger. */
 class Logger {
@@ -315,13 +314,6 @@ class Logger {
    */
   Status status(const Status& st);
 
-  /**
-   * Log a message from a Status object without returning it.
-   *
-   * @param st The Status object to log
-   */
-  void status_no_return_value(const Status& st);
-
   /**
    * Log an error and exit with a non-zero status.
    *
@@ -343,6 +335,46 @@ class Logger {
    */
   void fatal(const std::stringstream& msg);
 
+  /**
+   * Logs a message at the given level with no formatting.
+   *
+   * @param lvl The level of the log message.
+   * @param msg The message to log.
+   */
+  void log(const Level lvl, const char* msg);
+
+  /**
+   * Logs a message at the given level with no formatting.
+   *
+   * @param lvl The level of the log message.
+   * @param msg The message to log.
+   */
+  void log(const Level lvl, const std::string& msg);
+
+  /**
+   * Logs a message at the given level with no formatting.
+   *
+   * @param lvl The level of the log message.
+   * @param msg The message to log.
+   */
+  void log(const Level lvl, const std::stringstream& msg);
+
+  /**
+   * Logs a formatted message at the given level.
+   *
+   * @param lvl The level of the log message.
+   * @param msg The message to log.
+   */
+  template <typename... Args>
+  void log(const Level lvl, fmt::format_string<Args...> fmt, Args&&... args) {
+    // Check that this level is enabled to avoid needlessly formatting the
+    // string.
+    if (!should_log(lvl)) {
+      return;
+    }
+    log(lvl, fmt::format(fmt, std::forward<Args>(args)...));
+  }
+
   /**
    * Check whether events of a given level are logged.
    *
@@ -457,8 +489,7 @@ inline Status logger_format_from_string(
   return Status::Ok();
 }
 
-}  // namespace common
-}  // namespace tiledb
+}  // namespace tiledb::common
 
 // Also include the public-permissible logger functions here.
 #include "tiledb/common/logger_public.h"
