@@ -45,14 +45,14 @@
 #include "tiledb/sm/crypto/encryption_key.h"
 #include "tiledb/sm/fragment/fragment_info.h"
 #include "tiledb/sm/metadata/metadata.h"
-#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 
 using namespace tiledb::common;
 
 namespace tiledb::sm {
 
 class ArraySchema;
-class SchemaEvolution;
+class ArraySchemaEvolution;
+class Context;
 class FragmentMetadata;
 class MemoryTracker;
 enum class QueryType : uint8_t;
@@ -99,7 +99,8 @@ class OpenedArray {
       uint64_t timestamp_start,
       uint64_t timestamp_end_opened_at,
       bool is_remote)
-      : array_dir_(ArrayDirectory(resources, array_uri))
+      : resources_(resources)
+      , array_dir_(ArrayDirectory(resources, array_uri))
       , array_schema_latest_(nullptr)
       , metadata_(memory_tracker)
       , metadata_loaded_(false)
@@ -112,6 +113,11 @@ class OpenedArray {
         encryption_key_->set_key(encryption_type, key_bytes, key_length));
   }
 
+  /** Returns the context resources. */
+  inline ContextResources& resources() {
+    return resources_;
+  }
+
   /** Returns the array directory object. */
   inline const ArrayDirectory& array_directory() const {
     return array_dir_.value();
@@ -136,6 +142,8 @@ class OpenedArray {
   inline void set_array_schema_latest(
       const shared_ptr<ArraySchema>& array_schema) {
     array_schema_latest_ = array_schema;
+    // Update the latest schema in the map of all array schemas.
+    array_schemas_all_[array_schema->name()] = array_schema;
   }
 
   /** Returns all array schemas. */
@@ -208,6 +216,9 @@ class OpenedArray {
   }
 
  private:
+  /** The context resources. */
+  ContextResources& resources_;
+
   /** The array directory object for listing URIs. */
   optional<ArrayDirectory> array_dir_;
 
@@ -284,8 +295,8 @@ class Array {
 
   /** Constructor. */
   Array(
+      ContextResources& resources,
       const URI& array_uri,
-      StorageManager* storage_manager,
       ConsistencyController& cc = controller());
 
   /** Destructor. */
@@ -346,6 +357,20 @@ class Array {
     opened_array_->set_array_schemas_all(std::move(all_schemas));
   }
 
+  /**
+   * Evolve a TileDB array schema and store its new schema.
+   *
+   * @param resources The context resources.
+   * @param array_uri The uri of the array whose schema is to be evolved.
+   * @param schema_evolution The schema evolution.
+   * @param encryption_key The encryption key to use.
+   */
+  static void evolve_array_schema(
+      ContextResources& resources,
+      const URI& array_uri,
+      ArraySchemaEvolution* array_schema,
+      const EncryptionKey& encryption_key);
+
   /** Returns the array URI. */
   const URI& array_uri() const;
 
@@ -353,6 +378,20 @@ class Array {
    * serialization in pre TileDB 2.4 */
   const URI& array_uri_serialized() const;
 
+  /**
+   * Creates a TileDB array, storing its schema.
+   *
+   * @param resources The context resources.
+   * @param array_uri The URI of the array to be created.
+   * @param array_schema The array schema.
+   * @param encryption_key The encryption key to use.
+   */
+  static void create(
+      ContextResources& resources,
+      const URI& array_uri,
+      const shared_ptr<ArraySchema>& array_schema,
+      const EncryptionKey& encryption_key);
+
   /**
    * Opens the array for reading at a timestamp retrieved from the config
    * or for writing.
@@ -420,26 +459,78 @@ class Array {
   Status close();
 
   /**
-   * Deletes the Array data with given URI.
+   * Performs deletion of local fragments with the given parent URI,
+   * between the provided timestamps.
    *
-   * @param uri The uri of the Array whose data is to be deleted.
+   * @param resources The context resources.
+   * @param uri The uri of the Array whose fragments are to be deleted.
+   * @param timestamp_start The start timestamp at which to delete fragments.
+   * @param timestamp_end The end timestamp at which to delete fragments.
+   * @param array_dir An optional ArrayDirectory from which to delete fragments.
    *
-   * @pre The Array must be open for exclusive writes
+   * @section Maturity Notes
+   * This is legacy code, ported from StorageManager during its removal process.
+   * Its existence supports the non-static `delete_fragments` API below,
+   * performing the actual deletion of fragments. This function is slated for
+   * removal and should be directly integrated into the function below.
    */
-  void delete_array(const URI& uri);
+  static void delete_fragments(
+      ContextResources& resources,
+      const URI& uri,
+      uint64_t timestamp_start,
+      uint64_t timstamp_end,
+      std::optional<ArrayDirectory> array_dir = std::nullopt);
 
   /**
-   * Deletes the fragments from the Array with given URI.
+   * Handles local and remote deletion of fragments between the provided
+   * timestamps from an open array with the given URI.
    *
    * @param uri The uri of the Array whose fragments are to be deleted.
    * @param timestamp_start The start timestamp at which to delete fragments.
    * @param timestamp_end The end timestamp at which to delete fragments.
    *
    * @pre The Array must be open for exclusive writes
+   *
+   * @section Maturity Notes
+   * This API is an interim version of its final product, awaiting rewrite.
+   * As is, it handles the incoming URI and invokes the remote or local function
+   * call accordingly. The local, static function above is legacy code which
+   * exists only to support this function. A rewrite should integrate the two
+   * and remove the need for any static APIs.
    */
   void delete_fragments(
       const URI& uri, uint64_t timestamp_start, uint64_t timstamp_end);
 
+  /**
+   * Performs deletion of the local array with the given parent URI.
+   *
+   * @param resources The context resources.
+   * @param uri The uri of the Array whose data is to be deleted.
+   *
+   * @section Maturity Notes
+   * This is legacy code, ported from StorageManager during its removal process.
+   * Its existence supports the non-static `delete_array` API below,
+   * performing the actual deletion of array data. This function is slated for
+   * removal and should be directly integrated into the function below.
+   */
+  static void delete_array(ContextResources& resources, const URI& uri);
+
+  /**
+   * Handles local and remote deletion of an open array with the given URI.
+   *
+   * @param uri The uri of the Array whose data is to be deleted.
+   *
+   * @pre The Array must be open for exclusive writes
+   *
+   * @section Maturity Notes
+   * This API is an interim version of its final product, awaiting rewrite.
+   * As is, it handles the incoming URI and invokes the remote or local function
+   * call accordingly. The local, static function above is legacy code which
+   * exists only to support this function. A rewrite should integrate the two
+   * and remove the need for any static APIs.
+   */
+  void delete_array(const URI& uri);
+
   /**
    * Deletes the fragments with the given URIs from the Array with given URI.
    *
@@ -469,11 +560,23 @@ class Array {
   }
 
   /**
-   * Get the enumeration for the given name.
+   * Retrieves the encryption type.
+   *
+   * @param resources The context resources.
+   * @param uri The URI of the array.
+   * @param encryption_type Set to the encryption type.
+   */
+  static void encryption_type(
+      ContextResources& resources,
+      const URI& uri,
+      EncryptionType* encryption_type);
+
+  /**
+   * Get the enumeration for the given name from the latest array schema.
    *
    * This function retrieves the enumeration for the given name. If the
    * corresponding enumeration has not been loaded from storage it is
-   * loaded before this function returns.
+   * loaded and stored in the latest schema before this function returns.
    *
    * @param enumeration_name The name of the enumeration.
    * @return shared_ptr<const Enumeration> or nullptr on failure.
@@ -482,7 +585,18 @@ class Array {
       const std::string& enumeration_name);
 
   /**
-   * Get the enumerations with the given names.
+   * Load enumerations on all schemas for the array's opened timestamp range.
+   * This function will store all loaded enumerations into their corresponding
+   * schemas. The returned enumerations are provided as a convenience to the
+   * caller and can be discarded if loading the enumerations is sufficient.
+   *
+   * @return Map of schema names and a list of all loaded enumerations.
+   */
+  std::unordered_map<std::string, std::vector<shared_ptr<const Enumeration>>>
+  get_enumerations_all_schemas();
+
+  /**
+   * Get the enumerations with the given names from the latest array schema.
    *
    * This function retrieves the enumerations with the given names. If the
    * corresponding enumerations have not been loaded from storage they are
@@ -494,8 +608,14 @@ class Array {
   std::vector<shared_ptr<const Enumeration>> get_enumerations(
       const std::vector<std::string>& enumeration_names);
 
-  /** Load all enumerations for the array. */
-  void load_all_enumerations();
+  /**
+   * Load all enumerations for the array.
+   *
+   * @param all_schemas If true, enumerations will be loaded on all schemas
+   *    within the current opened timestamps on the array. If false, only load
+   *    enumerations on the latest array schema.
+   */
+  void load_all_enumerations(bool all_schemas = false);
 
   /**
    * Returns `true` if the array is empty at the time it is opened.
@@ -515,23 +635,6 @@ class Array {
   /** Retrieves the query type. Throws if the array is not open. */
   QueryType get_query_type() const;
 
-  /**
-   * Returns the max buffer size given a fixed-sized attribute/dimension and
-   * a subarray. Errors if the array is not open.
-   */
-  Status get_max_buffer_size(
-      const char* name, const void* subarray, uint64_t* buffer_size);
-
-  /**
-   * Returns the max buffer size given a var-sized attribute/dimension and
-   * a subarray. Errors if the array is not open.
-   */
-  Status get_max_buffer_size(
-      const char* name,
-      const void* subarray,
-      uint64_t* buffer_off_size,
-      uint64_t* buffer_val_size);
-
   /**
    * Returns a reference to the private encryption key.
    */
@@ -778,6 +881,82 @@ class Array {
    */
   void non_empty_domain(void* domain, bool* is_empty);
 
+  /**
+   * Returns the non-empty domain of the opened array on the given dimension.
+   * This is the union of the non-empty domains of the array fragments.
+   *
+   * @param idx The dimension index.
+   * @param domain The domain to be retrieved.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_from_index(unsigned idx, void* domain, bool* is_empty);
+
+  /**
+   * Returns the non-empty domain size of the opened array on the given
+   * dimension. This is the union of the non-empty domains of the array
+   * fragments. Applicable only to var-sized dimensions.
+   *
+   * @param idx The dimension index.
+   * @param start_size The size in bytes of the range start.
+   * @param end_size The size in bytes of the range end.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_var_size_from_index(
+      unsigned idx, uint64_t* start_size, uint64_t* end_size, bool* is_empty);
+
+  /**
+   * Returns the non-empty domain of the opened array on the given dimension.
+   * This is the union of the non-empty domains of the array fragments.
+   * Applicable only to var-sized dimensions.
+   *
+   * @param idx The dimension index.
+   * @param start The domain range start to set.
+   * @param end The domain range end to set.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_var_from_index(
+      unsigned idx, void* start, void* end, bool* is_empty);
+
+  /**
+   * Returns the non-empty domain of the opened array on the given dimension.
+   * This is the union of the non-empty domains of the array fragments.
+   *
+   * @param field_name The dimension name.
+   * @param domain The domain to be retrieved.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_from_name(
+      std::string_view field_name, void* domain, bool* is_empty);
+
+  /**
+   * Returns the non-empty domain size of the open array on the given dimension.
+   * This is the union of the non-empty domains of the array fragments.
+   * Applicable only to var-sized dimensions.
+   *
+   * @param field_name The dimension name.
+   * @param start_size The size in bytes of the range start.
+   * @param end_size The size in bytes of the range end.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_var_size_from_name(
+      std::string_view field_name,
+      uint64_t* start_size,
+      uint64_t* end_size,
+      bool* is_empty);
+
+  /**
+   * Returns the non-empty domain of the opened array on the given dimension.
+   * This is the union of the non-empty domains of the array fragments.
+   * Applicable only to var-sized dimensions.
+
+   * @param field_name The dimension name.
+   * @param start The domain range start to set.
+   * @param end The domain range end to set.
+   * @param is_empty `true` if the non-empty domain (and array) is empty.
+   */
+  void non_empty_domain_var_from_name(
+      std::string_view field_name, void* start, void* end, bool* is_empty);
+
   /**
    * Retrieves the array metadata object that is already loaded. If it's not yet
    * loaded it will be empty.
@@ -808,7 +987,11 @@ class Array {
   bool serialize_non_empty_domain() const;
 
   /**
-   * Checks the config to se if enumerations should be serialized on array open.
+   * Checks the config to see if enumerations should be serialized on array
+   * open.
+   *
+   * @return True if either `rest.load_enumerations_on_array_open` or
+   * `rest.load_enumerations_on_array_open_all_schemas` is set, else False.
    */
   bool serialize_enumerations() const;
 
@@ -857,11 +1040,26 @@ class Array {
     return resources_.rest_client();
   }
 
+  /**
+   * Upgrade a TileDB array to latest format version.
+   *
+   * @param resources The context resources.
+   * @param uri The URI of the array.
+   * @param config Configuration parameters for the upgrade
+   *     (`nullptr` means default, which will use the config associated with
+   *      this instance).
+   */
+  static void upgrade_version(
+      ContextResources& resources, const URI& uri, const Config& config);
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
 
+  /** TileDB Context Resources. */
+  ContextResources& resources_;
+
   /** The opened array that can be used by queries. */
   shared_ptr<OpenedArray> opened_array_;
 
@@ -921,25 +1119,9 @@ class Array {
    */
   optional<uint64_t> new_component_timestamp_;
 
-  /** TileDB storage manager. */
-  StorageManager* storage_manager_;
-
-  /** TileDB Context Resources. */
-  ContextResources& resources_;
-
   /** The array config. */
   Config config_;
 
-  /** Stores the max buffer sizes requested last time by the user .*/
-  std::unordered_map<std::string, std::pair<uint64_t, uint64_t>>
-      last_max_buffer_sizes_;
-
-  /**
-   * This is the last subarray used by the user to retrieve the
-   * max buffer sizes.
-   */
-  std::vector<uint8_t> last_max_buffer_sizes_subarray_;
-
   /** True if the array is remote (has `tiledb://` URI scheme). */
   bool remote_;
 
@@ -998,49 +1180,20 @@ class Array {
       std::unordered_map<std::string, shared_ptr<ArraySchema>>>
   open_for_reads_without_fragments();
 
-  /** Opens an array for writes.
+  /**
+   * Opens an array for writes.
    *
    * @param array The array to open.
-   * @return tuple of Status, latest ArraySchema and map of all array schemas
-   *        Status Ok on success, else error
+   * @return tuple of latest ArraySchema and map of all array schemas
    *        ArraySchema The array schema to be retrieved after the
    *          array is opened.
    *        ArraySchemaMap Map of all array schemas found keyed by name
    */
   tuple<
-      Status,
-      optional<shared_ptr<ArraySchema>>,
-      optional<std::unordered_map<std::string, shared_ptr<ArraySchema>>>>
+      shared_ptr<ArraySchema>,
+      std::unordered_map<std::string, shared_ptr<ArraySchema>>>
   open_for_writes();
 
-  /** Clears the cached max buffer sizes and subarray. */
-  void clear_last_max_buffer_sizes();
-
-  /**
-   * Computes the maximum buffer sizes for all attributes given a subarray,
-   * which are cached locally in the instance.
-   */
-  Status compute_max_buffer_sizes(const void* subarray);
-
-  /**
-   * Computes an upper bound on the buffer sizes required for a read
-   * query, for a given subarray and set of attributes. Note that
-   * the attributes are already set in `max_buffer_sizes`
-   *
-   * @param subarray The subarray to focus on. Note that it must have the same
-   *     underlying type as the array domain.
-   * @param max_buffer_sizes The buffer sizes to be retrieved. This is a map
-   * from the attribute (or coordinates) name to a pair of sizes (in bytes). For
-   * fixed-sized attributes, the second size is ignored. For var-sized
-   *     attributes, the first is the offsets size and the second is the
-   *     values size.
-   * @return Status
-   */
-  Status compute_max_buffer_sizes(
-      const void* subarray,
-      std::unordered_map<std::string, std::pair<uint64_t, uint64_t>>*
-          max_buffer_sizes_) const;
-
   /**
    * Load non-remote array metadata.
    */
@@ -1068,6 +1221,17 @@ class Array {
   void set_array_closed();
 };
 
+/**
+ * Loads an enumeration into a schema.
+ * Used to implement `tiledb_array_schema_get_enumeration*` APIs.
+ *
+ * @param ctx
+ * @param enmr_name the requested enumeration
+ * @param schema the target schema
+ */
+void load_enumeration_into_schema(
+    Context& ctx, const std::string& enmr_name, ArraySchema& array_schema);
+
 }  // namespace tiledb::sm
 
 #endif  // TILEDB_ARRAY_H
