@@ -262,6 +262,15 @@ class Enumeration {
     return cell_val_num_;
   }
 
+  /**
+   * Get the value map of the enumeration
+   * @return const tdb::pmr::unordered_map<std::string_view, uint64_t>& The
+   *        value map of the enumeration.
+   */
+  const tdb::pmr::unordered_map<std::string_view, uint64_t>& value_map() const {
+    return value_map_;
+  }
+
   /**
    * Get the cell_size of the enumeration
    *
@@ -345,13 +354,6 @@ class Enumeration {
    */
   uint64_t index_of(const void* data, uint64_t size) const;
 
-  /**
-   * Dump a textual representation of the Enumeration to the FILE
-   *
-   * @param out A file pointer to write to. If out is nullptr, use stdout
-   */
-  void dump(FILE* out) const;
-
  private:
   /* ********************************* */
   /*        PRIVATE CONSTRUCTORS       */
@@ -438,3 +440,7 @@ class Enumeration {
 }  // namespace tiledb::sm
 
 #endif  // TILEDB_DOMAIN_H
+
+/** Converts the filter into a string representation. */
+std::ostream& operator<<(
+    std::ostream& os, const tiledb::sm::Enumeration& enumeration);
