@@ -28,6 +28,8 @@
  * @section DESCRIPTION
  *
  * This file declares the TileDB C API for serialization.
+ * APIs defined in this header are currently unstable and subject to breaking
+ * changes between minor releases.
  */
 
 #ifndef TILEDB_SERIALIZATION_H
@@ -106,6 +108,7 @@ TILEDB_EXPORT int32_t tiledb_serialize_array(
  * @param client_side Allows to specify different behavior depending on who is
  * serializing, the client (1) or the Cloud server (0). This is sometimes needed
  * since they are both using the same Core library APIs for serialization.
+ * @param array_uri uri of the array to deserialize.
  * @param array Will be set to a newly allocated array.
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
@@ -114,6 +117,7 @@ TILEDB_EXPORT int32_t tiledb_deserialize_array(
     const tiledb_buffer_t* buffer,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
+    const char* array_uri,
     tiledb_array_t** array) TILEDB_NOEXCEPT;
 
 /**
@@ -414,7 +418,7 @@ TILEDB_EXPORT int32_t tiledb_deserialize_array_non_empty_domain_all_dimensions(
 /**
  * Serializes the array max buffer sizes information into the given buffer.
  *
- * @note The caller must free the returned `tiledb_buffer_t`.
+ * This API is deprecated and using it will always return an error.
  *
  * @param ctx The TileDB context.
  * @param array Array to which the subarray belongs to
@@ -422,9 +426,9 @@ TILEDB_EXPORT int32_t tiledb_deserialize_array_non_empty_domain_all_dimensions(
  * @param serialization_type Type of serialization to use
  * @param buffer Will be set to a newly allocated buffer containing the
  *      serialized max buffer sizes.
- * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ * @return `TILEDB_ERR`
  */
-TILEDB_EXPORT int32_t tiledb_serialize_array_max_buffer_sizes(
+TILEDB_DEPRECATED_EXPORT int32_t tiledb_serialize_array_max_buffer_sizes(
     tiledb_ctx_t* ctx,
     const tiledb_array_t* array,
     const void* subarray,
