@@ -33,6 +33,7 @@
 #include "tiledb/sm/consolidator/fragment_consolidator.h"
 #include "tiledb/common/logger.h"
 #include "tiledb/sm/array_schema/array_schema.h"
+#include "tiledb/sm/enums/array_type.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/query_status.h"
 #include "tiledb/sm/enums/query_type.h"
@@ -70,7 +71,8 @@ void FragmentConsolidationWorkspace::resize_buffers(
     stats::Stats* stats,
     const FragmentConsolidationConfig& config,
     const ArraySchema& array_schema,
-    std::unordered_map<std::string, uint64_t>& avg_cell_sizes) {
+    std::unordered_map<std::string, uint64_t>& avg_cell_sizes,
+    uint64_t total_buffers_budget) {
   auto timer_se = stats->start_timer("resize_buffers");
 
   // For easy reference
@@ -133,15 +135,12 @@ void FragmentConsolidationWorkspace::resize_buffers(
     buffer_weights.emplace_back(sizeof(uint64_t));
   }
 
-  // Use the old buffer size setting to see how much memory we would use.
-  auto buffer_num = buffer_weights.size();
-  uint64_t total_budget = config.total_buffer_size_;
-
   // If a user set the per-attribute buffer size configuration, we override
   // the use of the total_budget_size config setting for backwards
   // compatible behavior.
+  auto buffer_num = buffer_weights.size();
   if (config.buffer_size_ != 0) {
-    total_budget = config.buffer_size_ * buffer_num;
+    total_buffers_budget = config.buffer_size_ * buffer_num;
   }
 
   // Calculate the size of individual buffers by assigning a weight based
@@ -149,7 +148,8 @@ void FragmentConsolidationWorkspace::resize_buffers(
   auto total_weights = std::accumulate(
       buffer_weights.begin(), buffer_weights.end(), static_cast<size_t>(0));
 
-  uint64_t adjusted_budget = total_budget / total_weights * total_weights;
+  uint64_t adjusted_budget =
+      total_buffers_budget / total_weights * total_weights;
 
   if (buffer_num > buffers_.size()) {
     sizes_.resize(buffer_num);
@@ -188,8 +188,10 @@ void FragmentConsolidationWorkspace::resize_buffers(
 /* ****************************** */
 
 FragmentConsolidator::FragmentConsolidator(
-    const Config& config, StorageManager* storage_manager)
-    : Consolidator(storage_manager) {
+    ContextResources& resources,
+    const Config& config,
+    StorageManager* storage_manager)
+    : Consolidator(resources, storage_manager) {
   auto st = set_config(config);
   if (!st.ok()) {
     throw FragmentConsolidatorException(st.message());
@@ -210,15 +212,14 @@ Status FragmentConsolidator::consolidate(
   check_array_uri(array_name);
 
   // Open array for reading
-  auto array_for_reads{
-      make_shared<Array>(HERE(), URI(array_name), storage_manager_)};
-  RETURN_NOT_OK(array_for_reads->open_without_fragments(
+  auto array_for_reads{make_shared<Array>(HERE(), resources_, URI(array_name))};
+  throw_if_not_ok(array_for_reads->open_without_fragments(
       encryption_type, encryption_key, key_length));
 
   // Open array for writing
-  auto array_for_writes{make_shared<Array>(
-      HERE(), array_for_reads->array_uri(), storage_manager_)};
-  RETURN_NOT_OK(array_for_writes->open(
+  auto array_for_writes{
+      make_shared<Array>(HERE(), resources_, array_for_reads->array_uri())};
+  throw_if_not_ok(array_for_writes->open(
       QueryType::WRITE, encryption_type, encryption_key, key_length));
 
   // Disable consolidation with timestamps on older arrays.
@@ -233,7 +234,7 @@ Status FragmentConsolidator::consolidate(
   // must be fetched (even before `config_.timestamp_start_`),
   // to compute the anterior ND range that can help determine
   // which dense fragments are consolidatable.
-  FragmentInfo fragment_info(URI(array_name), storage_manager_->resources());
+  FragmentInfo fragment_info(URI(array_name), resources_);
   auto st = fragment_info.load(
       array_for_reads->array_directory(),
       config_.timestamp_start_,
@@ -306,7 +307,7 @@ Status FragmentConsolidator::consolidate(
 
   RETURN_NOT_OK_ELSE(
       array_for_reads->close(), throw_if_not_ok(array_for_writes->close()));
-  RETURN_NOT_OK(array_for_writes->close());
+  throw_if_not_ok(array_for_writes->close());
 
   stats_->add_counter("consolidate_step_num", step);
 
@@ -322,15 +323,14 @@ Status FragmentConsolidator::consolidate_fragments(
   auto timer_se = stats_->start_timer("consolidate_frags");
 
   // Open array for reading
-  auto array_for_reads{
-      make_shared<Array>(HERE(), URI(array_name), storage_manager_)};
-  RETURN_NOT_OK(array_for_reads->open_without_fragments(
+  auto array_for_reads{make_shared<Array>(HERE(), resources_, URI(array_name))};
+  throw_if_not_ok(array_for_reads->open_without_fragments(
       encryption_type, encryption_key, key_length));
 
   // Open array for writing
-  auto array_for_writes{make_shared<Array>(
-      HERE(), array_for_reads->array_uri(), storage_manager_)};
-  RETURN_NOT_OK(array_for_writes->open(
+  auto array_for_writes{
+      make_shared<Array>(HERE(), resources_, array_for_reads->array_uri())};
+  throw_if_not_ok(array_for_writes->open(
       QueryType::WRITE, encryption_type, encryption_key, key_length));
 
   // Disable consolidation with timestamps on older arrays.
@@ -345,7 +345,7 @@ Status FragmentConsolidator::consolidate_fragments(
   }
 
   // Get all fragment info
-  FragmentInfo fragment_info(URI(array_name), storage_manager_->resources());
+  FragmentInfo fragment_info(URI(array_name), resources_);
   auto st = fragment_info.load(
       array_for_reads->array_directory(),
       0,
@@ -363,7 +363,26 @@ Status FragmentConsolidator::consolidate_fragments(
   NDRange union_non_empty_domains;
   std::unordered_set<std::string> to_consolidate_set;
   for (auto& uri : fragment_uris) {
-    to_consolidate_set.emplace(uri);
+    if (uri.find('/') == std::string::npos) {
+      // The fragment URI is relative and does not contain the array URI.
+      to_consolidate_set.emplace(uri);
+    } else {
+      // The fragment URI is absolute and should contain the correct array URI.
+      URI fragment_uri(uri);
+      FragmentID frag_id(fragment_uri);
+
+      // Check for valid URI based on array format version.
+      auto fragments_dir = array_for_reads->array_directory().get_fragments_dir(
+          frag_id.array_format_version());
+      if (fragment_uri !=
+          fragments_dir.join_path(fragment_uri.last_path_part().c_str())) {
+        throw FragmentConsolidatorException(
+            "Failed request to consolidate an invalid fragment URI '" +
+            fragment_uri.to_string() + "' for array at '" +
+            URI(array_name).to_string() + "'");
+      }
+      to_consolidate_set.emplace(fragment_uri.last_path_part());
+    }
   }
 
   // Make sure all fragments to consolidate are present
@@ -384,8 +403,64 @@ Status FragmentConsolidator::consolidate_fragments(
   }
 
   if (count != fragment_uris.size()) {
-    return logger_->status(Status_ConsolidatorError(
-        "Cannot consolidate; Not all fragments could be found"));
+    throw FragmentConsolidatorException(
+        "Cannot consolidate; Found " + std::to_string(count) + " of " +
+        std::to_string(fragment_uris.size()) + " required fragments.");
+  }
+
+  // In case we have a dense array check that the fragments can be consolidated
+  // without data loss. More specifically, if the union of the non-empty domains
+  // of the fragments which are selected for consolidation (which is equal to
+  // the non-empty domain of the resulting consolidated fragment) overlaps with
+  // any fragment created prior to this subset, then the subset is marked as
+  // non-consolidatable. Therefore, empty regions in the non-emtpy
+  // domain of the consolidated fragment will be filled with special values.
+  // Those values may erroneously overwrite older valid cell values.
+  if (array_for_reads->array_schema_latest().array_type() == ArrayType::DENSE) {
+    // Search every other fragment in this array if any of them overlaps in
+    // ranges and its timestamp range falls between the range of the fragments
+    // to consolidate throw error
+
+    // First calculate the max timestamp among the fragments which are selected
+    // for consolidation and use it as an upper bound.
+    uint64_t max_timestamp = std::numeric_limits<uint64_t>::min();
+    for (const auto& item : to_consolidate) {
+      const auto& range = item.timestamp_range();
+      max_timestamp = std::max(max_timestamp, range.second);
+    }
+
+    // Expand domain to full tiles
+    auto expanded_union_non_empty_domains = union_non_empty_domains;
+    array_for_reads->array_schema_latest().current_domain().expand_to_tiles(
+        domain, expanded_union_non_empty_domains);
+
+    // Now iterate all fragments and see if the consolidation can lead to data
+    // loss
+    for (auto& frag_info : frag_info_vec) {
+      // Ignore the fragments that are requested to be consolidated
+      auto uri = frag_info.uri().last_path_part();
+      if (to_consolidate_set.count(uri) != 0) {
+        continue;
+      }
+
+      // Check domain and timestamp overlap. Do timestamp check first as it is
+      // cheaper. We compare the current fragment's start timestamp against the
+      // upper bound we calculated previously.
+      auto timestamp_range{frag_info.timestamp_range()};
+      bool timestamp_overlap = !(timestamp_range.first > max_timestamp);
+      if (timestamp_overlap &&
+          domain.overlap(
+              expanded_union_non_empty_domains, frag_info.non_empty_domain())) {
+        throw FragmentConsolidatorException(
+            "Cannot consolidate; The non-empty domain of the fragment with "
+            "URI: " +
+            uri +
+            " overlaps with the union of the non-empty domains of the "
+            "fragments selected for consolidation and was created before "
+            "these fragments. For more information refer to our "
+            "documentation on consolidation for Dense arrays.");
+      }
+    }
   }
 
   FragmentConsolidationWorkspace cw(consolidator_memory_tracker_);
@@ -430,7 +505,7 @@ void FragmentConsolidator::vacuum(const char* array_name) {
 
   // Get the fragment URIs and vacuum file URIs to be vacuumed
   ArrayDirectory array_dir(
-      storage_manager_->resources(),
+      resources_,
       URI(array_name),
       0,
       std::numeric_limits<uint64_t>::max(),
@@ -446,32 +521,31 @@ void FragmentConsolidator::vacuum(const char* array_name) {
     array_dir.write_commit_ignore_file(commit_uris_to_ignore);
   }
 
-  auto vfs = storage_manager_->vfs();
-  auto compute_tp = storage_manager_->compute_tp();
-
   // Delete fragment directories
+  auto& vfs = resources_.vfs();
+  auto& compute_tp = resources_.compute_tp();
   throw_if_not_ok(parallel_for(
-      compute_tp, 0, fragment_uris_to_vacuum.size(), [&](size_t i) {
+      &compute_tp, 0, fragment_uris_to_vacuum.size(), [&](size_t i) {
         // Remove the commit file, if present.
         auto commit_uri = array_dir.get_commit_uri(fragment_uris_to_vacuum[i]);
         bool is_file = false;
-        RETURN_NOT_OK(vfs->is_file(commit_uri, &is_file));
+        throw_if_not_ok(vfs.is_file(commit_uri, &is_file));
         if (is_file) {
-          RETURN_NOT_OK(vfs->remove_file(commit_uri));
+          throw_if_not_ok(vfs.remove_file(commit_uri));
         }
 
         bool is_dir = false;
-        RETURN_NOT_OK(vfs->is_dir(fragment_uris_to_vacuum[i], &is_dir));
+        throw_if_not_ok(vfs.is_dir(fragment_uris_to_vacuum[i], &is_dir));
         if (is_dir) {
-          RETURN_NOT_OK(vfs->remove_dir(fragment_uris_to_vacuum[i]));
+          throw_if_not_ok(vfs.remove_dir(fragment_uris_to_vacuum[i]));
         }
 
         return Status::Ok();
       }));
 
   // Delete the vacuum files.
-  vfs->remove_files(
-      compute_tp, filtered_fragment_uris.fragment_vac_uris_to_vacuum());
+  vfs.remove_files(
+      &compute_tp, filtered_fragment_uris.fragment_vac_uris_to_vacuum());
 }
 
 /* ****************************** */
@@ -553,25 +627,31 @@ Status FragmentConsolidator::consolidate_internal(
     }
   }
 
+  // Compute memory budgets
+  uint64_t total_weights =
+      config_.buffers_weight_ + config_.reader_weight_ + config_.writer_weight_;
+  uint64_t single_unit_budget = config_.total_budget_ / total_weights;
+  uint64_t buffers_budget = config_.buffers_weight_ * single_unit_budget;
+  uint64_t reader_budget = config_.reader_weight_ * single_unit_budget;
+  uint64_t writer_budget = config_.writer_weight_ * single_unit_budget;
+
   // Prepare buffers
   auto average_var_cell_sizes = array_for_reads->get_average_var_cell_sizes();
-  cw.resize_buffers(stats_, config_, array_schema, average_var_cell_sizes);
+  cw.resize_buffers(
+      stats_, config_, array_schema, average_var_cell_sizes, buffers_budget);
 
   // Create queries
-  auto query_r = (Query*)nullptr;
-  auto query_w = (Query*)nullptr;
-  auto st = create_queries(
+  tdb_unique_ptr<Query> query_r = nullptr;
+  tdb_unique_ptr<Query> query_w = nullptr;
+  throw_if_not_ok(create_queries(
       array_for_reads,
       array_for_writes,
       union_non_empty_domains,
-      &query_r,
-      &query_w,
-      new_fragment_uri);
-  if (!st.ok()) {
-    tdb_delete(query_r);
-    tdb_delete(query_w);
-    return st;
-  }
+      query_r,
+      query_w,
+      new_fragment_uri,
+      reader_budget,
+      writer_budget));
 
   // Get the vacuum URI
   URI vac_uri;
@@ -579,31 +659,20 @@ Status FragmentConsolidator::consolidate_internal(
     vac_uri =
         array_for_reads->array_directory().get_vacuum_uri(*new_fragment_uri);
   } catch (std::exception& e) {
-    tdb_delete(query_r);
-    tdb_delete(query_w);
-    std::throw_with_nested(
-        std::logic_error("[FragmentConsolidator::consolidate_internal] "));
+    FragmentConsolidatorException(
+        "Internal consolidation failed with exception" + std::string(e.what()));
   }
 
   // Read from one array and write to the other
-  try {
-    copy_array(query_r, query_w, cw);
-  } catch (...) {
-    tdb_delete(query_r);
-    tdb_delete(query_w);
-    throw;
-  };
+  copy_array(query_r.get(), query_w.get(), cw);
 
   // Finalize write query
-  st = query_w->finalize();
+  auto st = query_w->finalize();
   if (!st.ok()) {
-    tdb_delete(query_r);
-    tdb_delete(query_w);
     bool is_dir = false;
-    auto st2 = storage_manager_->vfs()->is_dir(*new_fragment_uri, &is_dir);
-    (void)st2;  // Perhaps report this once we support an error stack
+    throw_if_not_ok(resources_.vfs().is_dir(*new_fragment_uri, &is_dir));
     if (is_dir)
-      throw_if_not_ok(storage_manager_->vfs()->remove_dir(*new_fragment_uri));
+      throw_if_not_ok(resources_.vfs().remove_dir(*new_fragment_uri));
     return st;
   }
 
@@ -614,20 +683,13 @@ Status FragmentConsolidator::consolidate_internal(
       vac_uri,
       to_consolidate);
   if (!st.ok()) {
-    tdb_delete(query_r);
-    tdb_delete(query_w);
     bool is_dir = false;
-    throw_if_not_ok(
-        storage_manager_->vfs()->is_dir(*new_fragment_uri, &is_dir));
+    throw_if_not_ok(resources_.vfs().is_dir(*new_fragment_uri, &is_dir));
     if (is_dir)
-      throw_if_not_ok(storage_manager_->vfs()->remove_dir(*new_fragment_uri));
+      throw_if_not_ok(resources_.vfs().remove_dir(*new_fragment_uri));
     return st;
   }
 
-  // Clean up
-  tdb_delete(query_r);
-  tdb_delete(query_w);
-
   return st;
 }
 
@@ -667,9 +729,11 @@ Status FragmentConsolidator::create_queries(
     shared_ptr<Array> array_for_reads,
     shared_ptr<Array> array_for_writes,
     const NDRange& subarray,
-    Query** query_r,
-    Query** query_w,
-    URI* new_fragment_uri) {
+    tdb_unique_ptr<Query>& query_r,
+    tdb_unique_ptr<Query>& query_w,
+    URI* new_fragment_uri,
+    uint64_t read_memory_budget,
+    uint64_t write_memory_budget) {
   auto timer_se = stats_->start_timer("consolidate_create_queries");
 
   const auto dense = array_for_reads->array_schema_latest().dense();
@@ -679,48 +743,63 @@ Status FragmentConsolidator::create_queries(
   // is not a user input prone to errors).
 
   // Create read query
-  *query_r = tdb_new(Query, storage_manager_, array_for_reads);
-  RETURN_NOT_OK((*query_r)->set_layout(Layout::GLOBAL_ORDER));
+  query_r = tdb_unique_ptr<Query>(tdb_new(
+      Query,
+      resources_,
+      CancellationSource(storage_manager_),
+      storage_manager_,
+      array_for_reads,
+      nullopt,
+      read_memory_budget));
+  throw_if_not_ok(query_r->set_layout(Layout::GLOBAL_ORDER));
 
   // Dense consolidation will do a tile aligned read.
   if (dense) {
     NDRange read_subarray = subarray;
     auto& domain{array_for_reads->array_schema_latest().domain()};
-    domain.expand_to_tiles(&read_subarray);
-    RETURN_NOT_OK((*query_r)->set_subarray_unsafe(read_subarray));
+    array_for_reads->array_schema_latest().current_domain().expand_to_tiles(
+        domain, read_subarray);
+    throw_if_not_ok(query_r->set_subarray_unsafe(read_subarray));
   }
 
   // Enable consolidation with timestamps on the reader, if applicable.
   if (config_.with_timestamps_ && !dense) {
-    throw_if_not_ok((*query_r)->set_consolidation_with_timestamps());
+    throw_if_not_ok(query_r->set_consolidation_with_timestamps());
   }
 
   // Get last fragment URI, which will be the URI of the consolidated fragment
-  auto first = (*query_r)->first_fragment_uri();
-  auto last = (*query_r)->last_fragment_uri();
+  auto first = query_r->first_fragment_uri();
+  auto last = query_r->last_fragment_uri();
 
   auto write_version = array_for_reads->array_schema_latest().write_version();
   auto fragment_name = storage_format::generate_consolidated_fragment_name(
       first, last, write_version);
 
   // Create write query
-  *query_w = tdb_new(Query, storage_manager_, array_for_writes, fragment_name);
-  RETURN_NOT_OK((*query_w)->set_layout(Layout::GLOBAL_ORDER));
-  RETURN_NOT_OK((*query_w)->disable_checks_consolidation());
-  (*query_w)->set_fragment_size(config_.max_fragment_size_);
+  query_w = tdb_unique_ptr<Query>(tdb_new(
+      Query,
+      resources_,
+      CancellationSource(storage_manager_),
+      storage_manager_,
+      array_for_writes,
+      fragment_name,
+      write_memory_budget));
+  throw_if_not_ok(query_w->set_layout(Layout::GLOBAL_ORDER));
+  throw_if_not_ok(query_w->disable_checks_consolidation());
+  query_w->set_fragment_size(config_.max_fragment_size_);
   if (array_for_reads->array_schema_latest().dense()) {
-    RETURN_NOT_OK((*query_w)->set_subarray_unsafe(subarray));
+    throw_if_not_ok(query_w->set_subarray_unsafe(subarray));
   }
 
   // Set the processed conditions on new fragment.
   const auto& delete_and_update_tiles_location =
-      (*query_r)->array()->array_directory().delete_and_update_tiles_location();
+      query_r->array()->array_directory().delete_and_update_tiles_location();
   std::vector<std::string> processed_conditions;
   processed_conditions.reserve(delete_and_update_tiles_location.size());
   for (auto& location : delete_and_update_tiles_location) {
     processed_conditions.emplace_back(location.condition_marker());
   }
-  (*query_w)->set_processed_conditions(processed_conditions);
+  query_w->set_processed_conditions(processed_conditions);
 
   // Set the URI for the new fragment.
   auto frag_uri =
@@ -933,63 +1012,73 @@ void FragmentConsolidator::set_query_buffers(
 
 Status FragmentConsolidator::set_config(const Config& config) {
   // Set the consolidation config for ease of use
-  Config merged_config = storage_manager_->config();
+  Config merged_config = resources_.config();
   merged_config.inherit(config);
   bool found = false;
   config_.amplification_ = 0.0f;
-  RETURN_NOT_OK(merged_config.get<float>(
+  throw_if_not_ok(merged_config.get<float>(
       "sm.consolidation.amplification", &config_.amplification_, &found));
   assert(found);
   config_.steps_ = 0;
-  RETURN_NOT_OK(merged_config.get<uint32_t>(
+  throw_if_not_ok(merged_config.get<uint32_t>(
       "sm.consolidation.steps", &config_.steps_, &found));
   assert(found);
   config_.buffer_size_ = 0;
   // Only set the buffer_size_ if the user specified a value. Otherwise, we use
-  // the new sm.consolidation.total_buffer_size instead.
+  // the new sm.mem.consolidation.buffers_weight instead.
   if (merged_config.set_params().count("sm.consolidation.buffer_size") > 0) {
     logger_->warn(
         "The `sm.consolidation.buffer_size configuration setting has been "
         "deprecated. Set consolidation buffer sizes using the newer "
-        "`sm.consolidation.total_buffer_size` setting.");
-    RETURN_NOT_OK(merged_config.get<uint64_t>(
+        "`sm.mem.consolidation.buffers_weight` setting.");
+    throw_if_not_ok(merged_config.get<uint64_t>(
         "sm.consolidation.buffer_size", &config_.buffer_size_, &found));
     assert(found);
   }
-  config_.total_buffer_size_ = 0;
-  RETURN_NOT_OK(merged_config.get<uint64_t>(
-      "sm.consolidation.total_buffer_size",
-      &config_.total_buffer_size_,
-      &found));
+  config_.total_budget_ = 0;
+  throw_if_not_ok(merged_config.get<uint64_t>(
+      "sm.mem.total_budget", &config_.total_budget_, &found));
+  assert(found);
+  config_.buffers_weight_ = 0;
+  throw_if_not_ok(merged_config.get<uint64_t>(
+      "sm.mem.consolidation.buffers_weight", &config_.buffers_weight_, &found));
+  assert(found);
+  config_.reader_weight_ = 0;
+  throw_if_not_ok(merged_config.get<uint64_t>(
+      "sm.mem.consolidation.reader_weight", &config_.reader_weight_, &found));
+  assert(found);
+  config_.writer_weight_ = 0;
+  throw_if_not_ok(merged_config.get<uint64_t>(
+      "sm.mem.consolidation.writer_weight", &config_.writer_weight_, &found));
   assert(found);
   config_.max_fragment_size_ = 0;
-  RETURN_NOT_OK(merged_config.get<uint64_t>(
+  throw_if_not_ok(merged_config.get<uint64_t>(
       "sm.consolidation.max_fragment_size",
       &config_.max_fragment_size_,
       &found));
   assert(found);
   config_.size_ratio_ = 0.0f;
-  RETURN_NOT_OK(merged_config.get<float>(
+  throw_if_not_ok(merged_config.get<float>(
       "sm.consolidation.step_size_ratio", &config_.size_ratio_, &found));
   assert(found);
   config_.purge_deleted_cells_ = false;
-  RETURN_NOT_OK(merged_config.get<bool>(
+  throw_if_not_ok(merged_config.get<bool>(
       "sm.consolidation.purge_deleted_cells",
       &config_.purge_deleted_cells_,
       &found));
   assert(found);
   config_.min_frags_ = 0;
-  RETURN_NOT_OK(merged_config.get<uint32_t>(
+  throw_if_not_ok(merged_config.get<uint32_t>(
       "sm.consolidation.step_min_frags", &config_.min_frags_, &found));
   assert(found);
   config_.max_frags_ = 0;
-  RETURN_NOT_OK(merged_config.get<uint32_t>(
+  throw_if_not_ok(merged_config.get<uint32_t>(
       "sm.consolidation.step_max_frags", &config_.max_frags_, &found));
   assert(found);
-  RETURN_NOT_OK(merged_config.get<uint64_t>(
+  throw_if_not_ok(merged_config.get<uint64_t>(
       "sm.consolidation.timestamp_start", &config_.timestamp_start_, &found));
   assert(found);
-  RETURN_NOT_OK(merged_config.get<uint64_t>(
+  throw_if_not_ok(merged_config.get<uint64_t>(
       "sm.consolidation.timestamp_end", &config_.timestamp_end_, &found));
   assert(found);
   std::string reader =
@@ -1001,17 +1090,17 @@ Status FragmentConsolidator::set_config(const Config& config) {
 
   // Sanity checks
   if (config_.min_frags_ > config_.max_frags_)
-    return logger_->status(Status_ConsolidatorError(
+    throw FragmentConsolidatorException(
         "Invalid configuration; Minimum fragments config parameter is larger "
-        "than the maximum"));
+        "than the maximum");
   if (config_.size_ratio_ > 1.0f || config_.size_ratio_ < 0.0f)
-    return logger_->status(Status_ConsolidatorError(
+    throw FragmentConsolidatorException(
         "Invalid configuration; Step size ratio config parameter must be in "
-        "[0.0, 1.0]"));
+        "[0.0, 1.0]");
   if (config_.amplification_ < 0)
-    return logger_->status(
-        Status_ConsolidatorError("Invalid configuration; Amplification config "
-                                 "parameter must be non-negative"));
+    throw FragmentConsolidatorException(
+        "Invalid configuration; Amplification config parameter must be "
+        "non-negative");
 
   return Status::Ok();
 }
@@ -1035,9 +1124,8 @@ Status FragmentConsolidator::write_vacuum_file(
   }
 
   auto data = ss.str();
-  RETURN_NOT_OK(
-      storage_manager_->vfs()->write(vac_uri, data.c_str(), data.size()));
-  RETURN_NOT_OK(storage_manager_->vfs()->close_file(vac_uri));
+  throw_if_not_ok(resources_.vfs().write(vac_uri, data.c_str(), data.size()));
+  throw_if_not_ok(resources_.vfs().close_file(vac_uri));
 
   return Status::Ok();
 }
