@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,10 +39,11 @@
 
 #include "tiledb/common/rwlock.h"
 #include "tiledb/common/status.h"
-#include "tiledb/common/thread_pool.h"
+#include "tiledb/common/thread_pool/thread_pool.h"
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/config/config.h"
 #include "tiledb/sm/curl/curl_init.h"
+#include "tiledb/sm/filesystem/ls_scanner.h"
 #include "tiledb/sm/filesystem/ssl_config.h"
 #include "tiledb/sm/misc/constants.h"
 #include "uri.h"
@@ -69,6 +70,14 @@ class directory_entry;
 
 namespace sm {
 
+/** Class for GCS status exceptions. */
+class GCSException : public StatusException {
+ public:
+  explicit GCSException(const std::string& msg)
+      : StatusException("GCS", msg) {
+  }
+};
+
 class GCS {
  public:
   /* ********************************* */
@@ -224,6 +233,32 @@ class GCS {
       const std::string& delimiter = "/",
       int max_paths = -1) const;
 
+  /**
+   * Lists objects and object information that start with `prefix`, invoking
+   * the FilePredicate on each entry collected and the DirectoryPredicate on
+   * common prefixes for pruning.
+   *
+   * @param parent The parent prefix to list sub-paths.
+   * @param file_filter The FilePredicate to invoke on each object for
+   * filtering.
+   * @param directory_filter The DirectoryPredicate to invoke on each common
+   * prefix for pruning. This is currently unused, but is kept here for future
+   * support.
+   * @param recursive Whether to recursively list subdirectories.
+   * @return Vector of results with each entry being a pair of the string URI
+   * and object size.
+   */
+  template <FilePredicate F, DirectoryPredicate D>
+  LsObjects ls_filtered(
+      const URI& uri,
+      F file_filter,
+      [[maybe_unused]] D directory_filter = accept_all_dirs,
+      bool recursive = false) const {
+    // We use the constructor of std::function that accepts an F&& to convert
+    // the generic F to a polymorphic std::function.
+    return ls_filtered_impl(uri, std::move(file_filter), recursive);
+  }
+
   /**
    *
    * Lists objects and object information that start with `prefix`.
@@ -233,8 +268,7 @@ class GCS {
    * @param max_paths The maximum number of paths to be retrieved
    * @return A list of directory_entry objects
    */
-  tuple<Status, optional<std::vector<filesystem::directory_entry>>>
-  ls_with_sizes(
+  std::vector<filesystem::directory_entry> ls_with_sizes(
       const URI& uri,
       const std::string& delimiter = "/",
       int max_paths = -1) const;
@@ -372,7 +406,7 @@ class GCS {
     std::string next_part_path() {
       const uint64_t part_id = next_part_id_++;
       const std::string part_path =
-          object_path_ + "__tiledb_" + std::to_string(part_id);
+          object_path_ + ".part" + std::to_string(part_id) + ".tiledb.tmp";
       part_paths_.emplace_back(part_path);
       return part_path;
     }
@@ -599,6 +633,33 @@ class GCS {
    */
   Status is_bucket(const std::string& bucket_name, bool* const is_bucket) const;
 
+  /**
+   * Contains the implementation of ls_filtered.
+   *
+   * @section Notes
+   *
+   * The use of the non-generic std::function is necessary to keep the
+   * function's implementation in gcs.cc and avoid leaking the Google Cloud
+   * SDK headers, which would cause significant build performance regressions
+   * (see PR 4777). In the public-facing ls_filtered, we still use a generic
+   * callback which we convert.
+   *
+   * This has the consequence that the callback cannot capture variables that
+   * are not copy-constructible. It could be rectified with C++ 23's
+   * std::move_only_function, when it becomes available.
+   *
+   * @param uri The parent path to list sub-paths.
+   * @param file_filter The FilePredicate to invoke on each object for
+   * filtering.
+   * @param recursive Whether to recursively list subdirectories.
+   * @return Vector of results with each entry being a pair of the string URI
+   * and object size.
+   */
+  LsObjects ls_filtered_impl(
+      const URI& uri,
+      std::function<bool(const std::string_view, uint64_t)> file_filter,
+      bool recursive) const;
+
   /**
    * Thread-safe fetch of the write cache buffer in `write_cache_map_`.
    * If a buffer does not exist for `uri`, it will be created.
