@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,7 @@
 
 #include "vfs.h"
 #include "path_win.h"
+#include "tiledb/common/log_duration_instrument.h"
 #include "tiledb/common/logger_public.h"
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/enums/filesystem.h"
@@ -39,7 +40,6 @@
 #include "tiledb/sm/filesystem/hdfs_filesystem.h"
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/tdb_math.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/stats/global_stats.h"
 #include "tiledb/sm/tile/tile.h"
 
@@ -59,12 +59,14 @@ namespace tiledb::sm {
 
 VFS::VFS(
     stats::Stats* const parent_stats,
+    Logger* const logger,
     ThreadPool* const compute_tp,
     ThreadPool* const io_tp,
     const Config& config)
     : VFSBase(parent_stats)
     , S3_within_VFS(stats_, io_tp, config)
     , config_(config)
+    , logger_(logger)
     , compute_tp_(compute_tp)
     , io_tp_(io_tp)
     , vfs_params_(VFSParameters(config)) {
@@ -222,10 +224,8 @@ Status VFS::dir_size(const URI& dir_name, uint64_t* dir_size) const {
   do {
     auto uri = to_ls.front();
     to_ls.pop_front();
-    auto&& [st, children] = ls_with_sizes(uri);
-    RETURN_NOT_OK(st);
 
-    for (const auto& child : *children) {
+    for (const auto& child : ls_with_sizes(uri)) {
       if (!child.is_directory()) {
         *dir_size += child.file_size();
       } else {
@@ -238,6 +238,7 @@ Status VFS::dir_size(const URI& dir_name, uint64_t* dir_size) const {
 }
 
 Status VFS::touch(const URI& uri) const {
+  auto instrument = make_log_duration_instrument(uri, "touch");
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.touch(uri.to_path());
@@ -287,6 +288,7 @@ Status VFS::cancel_all_tasks() {
 }
 
 Status VFS::create_bucket(const URI& uri) const {
+  auto instrument = make_log_duration_instrument(uri, "create_bucket");
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     s3().create_bucket(uri);
@@ -339,6 +341,7 @@ Status VFS::remove_bucket(const URI& uri) const {
 }
 
 Status VFS::empty_bucket(const URI& uri) const {
+  auto instrument = make_log_duration_instrument(uri, "empty_bucket");
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     s3().empty_bucket(uri);
@@ -366,6 +369,7 @@ Status VFS::empty_bucket(const URI& uri) const {
 
 Status VFS::is_empty_bucket(
     const URI& uri, [[maybe_unused]] bool* is_empty) const {
+  auto instrument = make_log_duration_instrument(uri, "is_empty_bucket");
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     *is_empty = s3().is_empty_bucket(uri);
@@ -392,6 +396,7 @@ Status VFS::is_empty_bucket(
 }
 
 Status VFS::remove_dir(const URI& uri) const {
+  auto instrument = make_log_duration_instrument(uri, "remove_dir");
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.remove_dir(uri.to_path());
@@ -430,19 +435,31 @@ Status VFS::remove_dir(const URI& uri) const {
   return Status::Ok();
 }
 
+void VFS::remove_dir_if_empty(const URI& uri) const {
+  if (uri.is_file()) {
+#ifdef _WIN32
+    win_.remove_dir_if_empty(uri.to_path());
+#else
+    posix_.remove_dir_if_empty(uri.to_path());
+#endif
+  }
+  // Object stores do not have directories.
+}
+
 void VFS::remove_dirs(
     ThreadPool* compute_tp, const std::vector<URI>& uris) const {
   throw_if_not_ok(parallel_for(compute_tp, 0, uris.size(), [&](size_t i) {
     bool is_dir;
-    RETURN_NOT_OK(this->is_dir(uris[i], &is_dir));
+    throw_if_not_ok(this->is_dir(uris[i], &is_dir));
     if (is_dir) {
-      RETURN_NOT_OK(remove_dir(uris[i]));
+      throw_if_not_ok(remove_dir(uris[i]));
     }
     return Status::Ok();
   }));
 }
 
 Status VFS::remove_file(const URI& uri) const {
+  auto instrument = make_log_duration_instrument(uri, "remove_file");
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.remove_file(uri.to_path());
@@ -488,7 +505,7 @@ Status VFS::remove_file(const URI& uri) const {
 void VFS::remove_files(
     ThreadPool* compute_tp, const std::vector<URI>& uris) const {
   throw_if_not_ok(parallel_for(compute_tp, 0, uris.size(), [&](size_t i) {
-    RETURN_NOT_OK(remove_file(uris[i]));
+    throw_if_not_ok(remove_file(uris[i]));
     return Status::Ok();
   }));
 }
@@ -496,7 +513,7 @@ void VFS::remove_files(
 void VFS::remove_files(
     ThreadPool* compute_tp, const std::vector<TimestampedURI>& uris) const {
   throw_if_not_ok(parallel_for(compute_tp, 0, uris.size(), [&](size_t i) {
-    RETURN_NOT_OK(remove_file(uris[i].uri_));
+    throw_if_not_ok(remove_file(uris[i].uri_));
     return Status::Ok();
   }));
 }
@@ -528,6 +545,7 @@ Status VFS::max_parallel_ops(const URI& uri, uint64_t* ops) const {
 }
 
 Status VFS::file_size(const URI& uri, uint64_t* size) const {
+  auto instrument = make_log_duration_instrument(uri, "file_size");
   stats_->add_counter("file_size_num", 1);
   if (uri.is_file()) {
 #ifdef _WIN32
@@ -572,6 +590,7 @@ Status VFS::file_size(const URI& uri, uint64_t* size) const {
 }
 
 Status VFS::is_dir(const URI& uri, bool* is_dir) const {
+  auto instrument = make_log_duration_instrument(uri, "is_dir");
   if (uri.is_file()) {
 #ifdef _WIN32
     *is_dir = win_.is_dir(uri.to_path());
@@ -620,6 +639,7 @@ Status VFS::is_dir(const URI& uri, bool* is_dir) const {
 }
 
 Status VFS::is_file(const URI& uri, bool* is_file) const {
+  auto instrument = make_log_duration_instrument(uri, "is_file");
   stats_->add_counter("is_object_num", 1);
   if (uri.is_file()) {
 #ifdef _WIN32
@@ -670,6 +690,7 @@ Status VFS::is_file(const URI& uri, bool* is_file) const {
 }
 
 Status VFS::is_bucket(const URI& uri, bool* is_bucket) const {
+  auto instrument = make_log_duration_instrument(uri, "is_bucket");
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     *is_bucket = s3().is_bucket(uri);
@@ -703,93 +724,79 @@ Status VFS::is_bucket(const URI& uri, bool* is_bucket) const {
 
 Status VFS::ls(const URI& parent, std::vector<URI>* uris) const {
   stats_->add_counter("ls_num", 1);
-  auto&& [st, entries] = ls_with_sizes(parent);
-  RETURN_NOT_OK(st);
 
-  for (auto& fs : *entries) {
+  for (auto& fs : ls_with_sizes(parent)) {
     uris->emplace_back(fs.path().native());
   }
 
   return Status::Ok();
 }
 
-tuple<Status, optional<std::vector<directory_entry>>> VFS::ls_with_sizes(
-    const URI& parent) const {
+std::vector<directory_entry> VFS::ls_with_sizes(const URI& parent) const {
+  auto instrument = make_log_duration_instrument(parent, "ls");
   // Noop if `parent` is not a directory, do not error out.
   // For S3, GCS and Azure, `ls` on a non-directory will just
   // return an empty `uris` vector.
   if (!(parent.is_s3() || parent.is_gcs() || parent.is_azure())) {
     bool flag = false;
-    RETURN_NOT_OK_TUPLE(this->is_dir(parent, &flag), nullopt);
+    throw_if_not_ok(this->is_dir(parent, &flag));
 
     if (!flag) {
-      return {Status::Ok(), std::vector<directory_entry>()};
+      return {};
     }
   }
 
-  optional<std::vector<directory_entry>> entries;
+  std::vector<directory_entry> entries;
   if (parent.is_file()) {
 #ifdef _WIN32
-    Status st;
-    std::tie(st, entries) = win_.ls_with_sizes(parent);
+    entries = win_.ls_with_sizes(parent);
 #else
-    Status st;
-    std::tie(st, entries) = posix_.ls_with_sizes(parent);
+    entries = posix_.ls_with_sizes(parent);
 #endif
-    RETURN_NOT_OK_TUPLE(st, nullopt);
   } else if (parent.is_s3()) {
 #ifdef HAVE_S3
-    Status st;
-    std::tie(st, entries) = s3().ls_with_sizes(parent);
+    entries = s3().ls_with_sizes(parent);
 #else
-    auto st = Status_VFSError("TileDB was built without S3 support");
+    throw BuiltWithout("S3");
 #endif
-    RETURN_NOT_OK_TUPLE(st, nullopt);
   } else if (parent.is_azure()) {
 #ifdef HAVE_AZURE
-    Status st;
-    std::tie(st, entries) = azure_.ls_with_sizes(parent);
+    entries = azure_.ls_with_sizes(parent);
 #else
-    auto st = Status_VFSError("TileDB was built without Azure support");
+    throw BuiltWithout("Azure");
 #endif
-    RETURN_NOT_OK_TUPLE(st, nullopt);
   } else if (parent.is_gcs()) {
 #ifdef HAVE_GCS
-    Status st;
-    std::tie(st, entries) = gcs_.ls_with_sizes(parent);
+    entries = gcs_.ls_with_sizes(parent);
 #else
-    auto st = Status_VFSError("TileDB was built without GCS support");
+    throw BuiltWithout("GCS");
 #endif
-    RETURN_NOT_OK_TUPLE(st, nullopt);
   } else if (parent.is_hdfs()) {
 #ifdef HAVE_HDFS
-    Status st;
-    std::tie(st, entries) = hdfs_->ls_with_sizes(parent);
+    auto&& [st, entries_optional] = hdfs_->ls_with_sizes(parent);
+    throw_if_not_ok(st);
+    entries = *std::move(entries_optional);
 #else
-    auto st = Status_VFSError("TileDB was built without HDFS support");
+    throw BuiltWithout("HDFS");
 #endif
-    RETURN_NOT_OK_TUPLE(st, nullopt);
   } else if (parent.is_memfs()) {
-    Status st;
-    std::tie(st, entries) =
-        memfs_.ls_with_sizes(URI("mem://" + parent.to_path()));
-    RETURN_NOT_OK_TUPLE(st, nullopt);
+    entries = memfs_.ls_with_sizes(URI("mem://" + parent.to_path()));
   } else {
-    auto st = Status_VFSError("Unsupported URI scheme: " + parent.to_string());
-    return {st, std::nullopt};
+    throw UnsupportedURI(parent.to_string());
   }
   parallel_sort(
       compute_tp_,
-      entries->begin(),
-      entries->end(),
+      entries.begin(),
+      entries.end(),
       [](const directory_entry& l, const directory_entry& r) {
         return l.path().native() < r.path().native();
       });
 
-  return {Status::Ok(), entries};
+  return entries;
 }
 
 Status VFS::move_file(const URI& old_uri, const URI& new_uri) {
+  auto instrument = make_log_duration_instrument(old_uri, new_uri);
   // If new_uri exists, delete it or raise an error based on `force`
   bool is_file;
   RETURN_NOT_OK(this->is_file(new_uri, &is_file));
@@ -866,6 +873,7 @@ Status VFS::move_file(const URI& old_uri, const URI& new_uri) {
 }
 
 Status VFS::move_dir(const URI& old_uri, const URI& new_uri) {
+  auto instrument = make_log_duration_instrument(old_uri, new_uri);
   // File
   if (old_uri.is_file()) {
     if (new_uri.is_file()) {
@@ -938,6 +946,7 @@ Status VFS::move_dir(const URI& old_uri, const URI& new_uri) {
 }
 
 Status VFS::copy_file(const URI& old_uri, const URI& new_uri) {
+  auto instrument = make_log_duration_instrument(old_uri, new_uri);
   // If new_uri exists, delete it or raise an error based on `force`
   bool is_file;
   RETURN_NOT_OK(this->is_file(new_uri, &is_file));
@@ -1011,6 +1020,7 @@ Status VFS::copy_file(const URI& old_uri, const URI& new_uri) {
 }
 
 Status VFS::copy_dir(const URI& old_uri, const URI& new_uri) {
+  auto instrument = make_log_duration_instrument(old_uri, new_uri);
   // File
   if (old_uri.is_file()) {
     if (new_uri.is_file()) {
@@ -1146,6 +1156,7 @@ Status VFS::read_impl(
     void* const buffer,
     const uint64_t nbytes,
     [[maybe_unused]] const bool use_read_ahead) {
+  auto instrument = make_log_duration_instrument(uri, "read");
   stats_->add_counter("read_ops_num", 1);
   log_read(uri, offset, nbytes);
 
@@ -1306,6 +1317,7 @@ bool VFS::supports_uri_scheme(const URI& uri) const {
 }
 
 Status VFS::sync(const URI& uri) {
+  auto instrument = make_log_duration_instrument(uri, "sync");
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.sync(uri.to_path());
@@ -1397,6 +1409,7 @@ Status VFS::open_file(const URI& uri, VFSMode mode) {
 }
 
 Status VFS::close_file(const URI& uri) {
+  auto instrument = make_log_duration_instrument(uri, "close_file");
   if (uri.is_file()) {
 #ifdef _WIN32
     return win_.sync(uri.to_path());
@@ -1456,6 +1469,7 @@ Status VFS::write(
     const void* buffer,
     uint64_t buffer_size,
     [[maybe_unused]] bool remote_global_order_write) {
+  auto instrument = make_log_duration_instrument(uri, "write");
   stats_->add_counter("write_byte_num", buffer_size);
   stats_->add_counter("write_ops_num", 1);
 
@@ -1601,6 +1615,8 @@ Status VFS::set_multipart_upload_state(
 }
 
 Status VFS::flush_multipart_file_buffer(const URI& uri) {
+  auto instrument =
+      make_log_duration_instrument(uri, "flush_multipart_file_buffer");
   if (uri.is_s3()) {
 #ifdef HAVE_S3
     Buffer* buff = nullptr;
@@ -1662,4 +1678,30 @@ void VFS::log_read(const URI& uri, uint64_t offset, uint64_t nbytes) {
   LOG_INFO("VFS Read: " + read_to_log);
 }
 
+optional<LogDurationInstrument> VFS::make_log_duration_instrument(
+    const URI& uri, const std::string_view operation_name) const {
+  if (!vfs_params_.log_operations_ || logger_ == nullptr) {
+    return nullopt;
+  }
+  // Construct LogDurationInstrument in-place to take advantage of RVO.
+  return std::make_optional<LogDurationInstrument>(
+      logger_, "{} on {}", operation_name, uri.to_string());
+}
+
+optional<LogDurationInstrument> VFS::make_log_duration_instrument(
+    const URI& source,
+    const URI& destination,
+    const std::string_view operation_name) const {
+  if (!vfs_params_.log_operations_ || logger_ == nullptr) {
+    return nullopt;
+  }
+  // Construct LogDurationInstrument in-place to take advantage of RVO.
+  return std::make_optional<LogDurationInstrument>(
+      logger_,
+      "{} from {} to {}",
+      operation_name,
+      source.to_string(),
+      destination.to_string());
+}
+
 }  // namespace tiledb::sm
