@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -41,24 +41,21 @@
 #include "tiledb/sm/misc/hash.h"
 #include "tiledb/sm/misc/hilbert.h"
 #include "tiledb/sm/misc/parallel_functions.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/hilbert_order.h"
 #include "tiledb/sm/query/query_macros.h"
 #include "tiledb/sm/query/readers/result_tile.h"
 #include "tiledb/sm/stats/global_stats.h"
-#include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/subarray/subarray.h"
 
 using namespace tiledb;
 using namespace tiledb::common;
 using namespace tiledb::sm::stats;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
-class SparseGlobalOrderReaderStatusException : public StatusException {
+class SparseGlobalOrderReaderException : public StatusException {
  public:
-  explicit SparseGlobalOrderReaderStatusException(const std::string& message)
+  explicit SparseGlobalOrderReaderException(const std::string& message)
       : StatusException("SparseGlobalOrderReader", message) {
   }
 };
@@ -248,16 +245,14 @@ void SparseGlobalOrderReader<BitmapType>::load_all_tile_offsets() {
     // Make sure we have enough space for tile offsets data.
     uint64_t total_tile_offset_usage =
         tile_offsets_size(subarray_.relevant_fragments());
-    uint64_t available_memory =
-        array_memory_tracker_->get_memory_available() -
-        array_memory_tracker_->get_memory_usage(MemoryType::TILE_OFFSETS);
+    uint64_t available_memory = array_memory_tracker_->get_memory_available();
     if (total_tile_offset_usage > available_memory) {
-      throw SparseGlobalOrderReaderStatusException(
+      throw SparseGlobalOrderReaderException(
           "Cannot load tile offsets, computed size (" +
           std::to_string(total_tile_offset_usage) +
           ") is larger than available memory (" +
           std::to_string(available_memory) +
-          "). Total budget for array data (" +
+          "), increase memory budget. Total budget for array data (" +
           std::to_string(array_memory_tracker_->get_memory_budget()) + ").");
     }
 
@@ -351,9 +346,6 @@ SparseGlobalOrderReader<BitmapType>::create_result_tiles(
   // Get the number of fragments to process and compute per fragment memory.
   uint64_t num_fragments_to_process =
       tmp_read_state_.num_fragments_to_process();
-  per_fragment_memory_ = memory_budget_.total_budget() *
-                         memory_budget_.ratio_coords() /
-                         num_fragments_to_process;
 
   // Save which result tile list is empty.
   std::vector<uint64_t> rt_list_num_tiles(result_tiles.size());
@@ -361,16 +353,76 @@ SparseGlobalOrderReader<BitmapType>::create_result_tiles(
     rt_list_num_tiles[i] = result_tiles[i].size();
   }
 
-  // Create result tiles.
-  if (subarray_.is_set()) {
-    // Load as many tiles as the memory budget allows.
-    throw_if_not_ok(parallel_for(
-        storage_manager_->compute_tp(), 0, fragment_num, [&](uint64_t f) {
-          uint64_t t = 0;
-          auto& tile_ranges = tmp_read_state_.tile_ranges(f);
-          while (!tile_ranges.empty()) {
-            auto& range = tile_ranges.back();
-            for (t = range.first; t <= range.second; t++) {
+  if (num_fragments_to_process > 0) {
+    per_fragment_memory_ =
+        memory_budget_.coordinates_budget() / num_fragments_to_process;
+
+    // Create result tiles.
+    if (subarray_.is_set()) {
+      // Load as many tiles as the memory budget allows.
+      throw_if_not_ok(parallel_for(
+          &resources_.compute_tp(), 0, fragment_num, [&](uint64_t f) {
+            uint64_t t = 0;
+            auto& tile_ranges = tmp_read_state_.tile_ranges(f);
+            while (!tile_ranges.empty()) {
+              auto& range = tile_ranges.back();
+              for (t = range.first; t <= range.second; t++) {
+                auto budget_exceeded = add_result_tile(
+                    dim_num,
+                    per_fragment_memory_,
+                    f,
+                    t,
+                    *fragment_metadata_[f],
+                    result_tiles);
+
+                if (budget_exceeded) {
+                  logger_->debug(
+                      "Budget exceeded adding result tiles, fragment {0}, tile "
+                      "{1}",
+                      f,
+                      t);
+
+                  if (result_tiles[f].empty()) {
+                    auto tiles_size = get_coord_tiles_size(dim_num, f, t);
+                    throw SparseGlobalOrderReaderException(
+                        "Cannot load a single tile for fragment, increase "
+                        "memory "
+                        "budget, tile size : " +
+                        std::to_string(tiles_size) + ", per fragment memory " +
+                        std::to_string(per_fragment_memory_) +
+                        ", total budget " +
+                        std::to_string(memory_budget_.total_budget()) +
+                        ", processing fragment " + std::to_string(f) +
+                        " out of " + std::to_string(num_fragments_to_process) +
+                        " total fragments");
+                  }
+                  return Status::Ok();
+                }
+
+                range.first++;
+              }
+
+              tmp_read_state_.remove_tile_range(f);
+            }
+
+            tmp_read_state_.set_all_tiles_loaded(f);
+
+            return Status::Ok();
+          }));
+    } else {
+      // Load as many tiles as the memory budget allows.
+      throw_if_not_ok(parallel_for(
+          &resources_.compute_tp(), 0, fragment_num, [&](uint64_t f) {
+            uint64_t t = 0;
+            auto tile_num = fragment_metadata_[f]->tile_num();
+
+            // Figure out the start index.
+            auto start = read_state_.frag_idx()[f].tile_idx_;
+            if (!result_tiles[f].empty()) {
+              start = std::max(start, result_tiles[f].back().tile_idx() + 1);
+            }
+
+            for (t = start; t < tile_num; t++) {
               auto budget_exceeded = add_result_tile(
                   dim_num,
                   per_fragment_memory_,
@@ -388,77 +440,24 @@ SparseGlobalOrderReader<BitmapType>::create_result_tiles(
 
                 if (result_tiles[f].empty()) {
                   auto tiles_size = get_coord_tiles_size(dim_num, f, t);
-                  throw SparseGlobalOrderReaderStatusException(
-                      "Cannot load a single tile for fragment, increase "
-                      "memory "
+                  return logger_->status(Status_SparseGlobalOrderReaderError(
+                      "Cannot load a single tile for fragment, increase memory "
                       "budget, tile size : " +
                       std::to_string(tiles_size) + ", per fragment memory " +
                       std::to_string(per_fragment_memory_) + ", total budget " +
                       std::to_string(memory_budget_.total_budget()) +
                       ", num fragments to process " +
-                      std::to_string(num_fragments_to_process));
+                      std::to_string(num_fragments_to_process)));
                 }
                 return Status::Ok();
               }
-
-              range.first++;
             }
 
-            tmp_read_state_.remove_tile_range(f);
-          }
+            tmp_read_state_.set_all_tiles_loaded(f);
 
-          tmp_read_state_.set_all_tiles_loaded(f);
-
-          return Status::Ok();
-        }));
-  } else {
-    // Load as many tiles as the memory budget allows.
-    throw_if_not_ok(parallel_for(
-        storage_manager_->compute_tp(), 0, fragment_num, [&](uint64_t f) {
-          uint64_t t = 0;
-          auto tile_num = fragment_metadata_[f]->tile_num();
-
-          // Figure out the start index.
-          auto start = read_state_.frag_idx()[f].tile_idx_;
-          if (!result_tiles[f].empty()) {
-            start = std::max(start, result_tiles[f].back().tile_idx() + 1);
-          }
-
-          for (t = start; t < tile_num; t++) {
-            auto budget_exceeded = add_result_tile(
-                dim_num,
-                per_fragment_memory_,
-                f,
-                t,
-                *fragment_metadata_[f],
-                result_tiles);
-
-            if (budget_exceeded) {
-              logger_->debug(
-                  "Budget exceeded adding result tiles, fragment {0}, tile "
-                  "{1}",
-                  f,
-                  t);
-
-              if (result_tiles[f].empty()) {
-                auto tiles_size = get_coord_tiles_size(dim_num, f, t);
-                return logger_->status(Status_SparseGlobalOrderReaderError(
-                    "Cannot load a single tile for fragment, increase memory "
-                    "budget, tile size : " +
-                    std::to_string(tiles_size) + ", per fragment memory " +
-                    std::to_string(per_fragment_memory_) + ", total budget " +
-                    std::to_string(memory_budget_.total_budget()) +
-                    ", num fragments to process " +
-                    std::to_string(num_fragments_to_process)));
-              }
-              return Status::Ok();
-            }
-          }
-
-          tmp_read_state_.set_all_tiles_loaded(f);
-
-          return Status::Ok();
-        }));
+            return Status::Ok();
+          }));
+    }
   }
 
   bool done_adding_result_tiles = tmp_read_state_.done_adding_result_tiles();
@@ -493,8 +492,8 @@ void SparseGlobalOrderReader<BitmapType>::clean_tile_list(
     std::vector<ResultTilesList>& result_tiles) {
   // Clear result tiles that are not necessary anymore.
   auto fragment_num = fragment_metadata_.size();
-  throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(), 0, fragment_num, [&](uint64_t f) {
+  throw_if_not_ok(
+      parallel_for(&resources_.compute_tp(), 0, fragment_num, [&](uint64_t f) {
         auto it = result_tiles[f].begin();
         while (it != result_tiles[f].end()) {
           if (it->result_num() == 0) {
@@ -522,7 +521,7 @@ void SparseGlobalOrderReader<BitmapType>::dedup_tiles_with_timestamps(
 
   // Process all tiles in parallel.
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(), 0, result_tiles.size(), [&](uint64_t t) {
+      &resources_.compute_tp(), 0, result_tiles.size(), [&](uint64_t t) {
         const auto f = result_tiles[t]->frag_idx();
         if (fragment_metadata_[f]->has_timestamps()) {
           // For easy reference.
@@ -597,8 +596,8 @@ void SparseGlobalOrderReader<BitmapType>::dedup_fragments_with_timestamps(
 
   // Run all fragments in parallel.
   auto fragment_num = fragment_metadata_.size();
-  throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(), 0, fragment_num, [&](uint64_t f) {
+  throw_if_not_ok(
+      parallel_for(&resources_.compute_tp(), 0, fragment_num, [&](uint64_t f) {
         // Run only for fragments with timestamps.
         if (fragment_metadata_[f]->has_timestamps()) {
           // Process all tiles.
@@ -848,7 +847,7 @@ void SparseGlobalOrderReader<BitmapType>::compute_hilbert_values(
 
   // Parallelize on tiles.
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(), 0, result_tiles.size(), [&](uint64_t t) {
+      &resources_.compute_tp(), 0, result_tiles.size(), [&](uint64_t t) {
         auto tile =
             static_cast<GlobalOrderResultTile<BitmapType>*>(result_tiles[t]);
         auto cell_num =
@@ -928,7 +927,7 @@ SparseGlobalOrderReader<BitmapType>::merge_result_cell_slabs(
   // For all fragments, get the first tile in the sorting queue.
   std::vector<TileListIt> to_delete;
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(), 0, result_tiles.size(), [&](uint64_t f) {
+      &resources_.compute_tp(), 0, result_tiles.size(), [&](uint64_t f) {
         if (result_tiles[f].size() > 0) {
           // Initialize the iterator for this fragment.
           rt_it[f] = result_tiles[f].begin();
@@ -1175,7 +1174,7 @@ void SparseGlobalOrderReader<BitmapType>::copy_offsets_tiles(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_cell_slabs.size(),
       0,
@@ -1281,7 +1280,7 @@ void SparseGlobalOrderReader<BitmapType>::copy_var_data_tiles(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_cell_slabs.size(),
       0,
@@ -1352,7 +1351,7 @@ void SparseGlobalOrderReader<BitmapType>::copy_fixed_data_tiles(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_cell_slabs.size(),
       0,
@@ -1452,7 +1451,7 @@ void SparseGlobalOrderReader<BitmapType>::copy_timestamps_tiles(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_cell_slabs.size(),
       0,
@@ -1519,7 +1518,7 @@ void SparseGlobalOrderReader<BitmapType>::copy_delete_meta_tiles(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_cell_slabs.size(),
       0,
@@ -1583,6 +1582,7 @@ void SparseGlobalOrderReader<BitmapType>::copy_delete_meta_tiles(
               if (*src_buff_condition_indexes !=
                   std::numeric_limits<uint64_t>::max()) {
                 auto& condition_marker = fragment_metadata_[rt->frag_idx()]
+                                             ->loaded_metadata()
                                              ->get_processed_conditions()
                                                  [*src_buff_condition_indexes];
                 converted_index =
@@ -1627,8 +1627,8 @@ SparseGlobalOrderReader<BitmapType>::respect_copy_memory_budget(
   uint64_t max_cs_idx = result_cell_slabs.size();
   std::mutex max_cs_idx_mtx;
   std::vector<uint64_t> total_mem_usage_per_attr(names.size());
-  throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(), 0, names.size(), [&](uint64_t i) {
+  throw_if_not_ok(
+      parallel_for(&resources_.compute_tp(), 0, names.size(), [&](uint64_t i) {
         // For easy reference.
         const auto& name = names[i];
         const bool agg_only = aggregate_only(name);
@@ -1653,15 +1653,18 @@ SparseGlobalOrderReader<BitmapType>::respect_copy_memory_budget(
         // Get the size for all tiles.
         uint64_t idx = 0;
         for (; idx < max_cs_idx; idx++) {
-          // Skip this tile if it's aggregate only and we can aggregate it with
-          // the fragment metadata only.
-          if (agg_only &&
-              can_aggregate_tile_with_frag_md(result_cell_slabs[idx])) {
+          auto rcs = result_cell_slabs[idx];
+          if (rcs.length_ == 0) {
             continue;
           }
 
-          auto rt = static_cast<GlobalOrderResultTile<BitmapType>*>(
-              result_cell_slabs[idx].tile_);
+          // Skip this tile if it's aggregate only and we can aggregate it with
+          // the fragment metadata only.
+          if (agg_only && can_aggregate_tile_with_frag_md(rcs)) {
+            continue;
+          }
+
+          auto rt = static_cast<GlobalOrderResultTile<BitmapType>*>(rcs.tile_);
           const auto f = rt->frag_idx();
           const auto t = rt->tile_idx();
           auto id = std::pair<uint64_t, uint64_t>(f, t);
@@ -1710,7 +1713,7 @@ SparseGlobalOrderReader<BitmapType>::respect_copy_memory_budget(
       }));
 
   if (max_cs_idx == 0) {
-    throw SparseGlobalOrderReaderStatusException(
+    throw SparseGlobalOrderReaderException(
         "Unable to copy one slab with current budget/buffers");
   }
 
@@ -1838,7 +1841,7 @@ void SparseGlobalOrderReader<BitmapType>::process_slabs(
 
   // Compute parallelization parameters.
   uint64_t num_range_threads = 1;
-  const auto num_threads = storage_manager_->compute_tp()->concurrency_level();
+  const auto num_threads = resources_.compute_tp().concurrency_level();
   if (result_cell_slabs.size() < num_threads) {
     // Ceil the division between thread_num and tile_num.
     num_range_threads = 1 + ((num_threads - 1) / result_cell_slabs.size());
@@ -1947,7 +1950,7 @@ void SparseGlobalOrderReader<BitmapType>::process_slabs(
       if (aggregates_.count(name) != 0) {
         for (auto& aggregates : aggregates_[name]) {
           if (aggregates->need_recompute_on_overflow()) {
-            throw SparseGlobalOrderReaderStatusException(
+            throw SparseGlobalOrderReaderException(
                 "Overflow happened after aggregate was computed, aggregate "
                 "recompute pass is not yet implemented");
           }
@@ -2120,7 +2123,7 @@ void SparseGlobalOrderReader<BitmapType>::process_aggregates(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_cell_slabs.size(),
       0,
@@ -2195,8 +2198,8 @@ void SparseGlobalOrderReader<BitmapType>::end_iteration(
   auto fragment_num = fragment_metadata_.size();
 
   // Clear fully processed tiles in each fragments.
-  throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(), 0, fragment_num, [&](uint64_t f) {
+  throw_if_not_ok(
+      parallel_for(&resources_.compute_tp(), 0, fragment_num, [&](uint64_t f) {
         while (!result_tiles[f].empty() &&
                result_tiles[f].front().tile_idx() <
                    read_state_.frag_idx()[f].tile_idx_) {
@@ -2227,5 +2230,4 @@ template SparseGlobalOrderReader<uint8_t>::SparseGlobalOrderReader(
 template SparseGlobalOrderReader<uint64_t>::SparseGlobalOrderReader(
     stats::Stats*, shared_ptr<Logger>, StrategyParams&, bool);
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
