@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2022 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,7 @@
 #include "tiledb/common/logger.h"
 #include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/array/array.h"
+#include "tiledb/sm/array/array_operations.h"
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/filesystem/vfs.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
@@ -49,12 +50,11 @@
 
 #include <numeric>
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
-class SparseIndexReaderBaseStatusException : public StatusException {
+class SparseIndexReaderBaseException : public StatusException {
  public:
-  explicit SparseIndexReaderBaseStatusException(const std::string& message)
+  explicit SparseIndexReaderBaseException(const std::string& message)
       : StatusException("SparseIndexReaderBase", message) {
   }
 };
@@ -72,21 +72,14 @@ SparseIndexReaderBase::SparseIndexReaderBase(
     : ReaderBase(stats, logger, params)
     , read_state_(array_->fragment_metadata().size())
     , tmp_read_state_(array_->fragment_metadata().size())
-    , memory_budget_(config_, reader_string)
+    , memory_budget_(config_, reader_string, params.memory_budget())
     , include_coords_(include_coords)
     , memory_used_for_coords_total_(0)
-    , deletes_consolidation_no_purge_(
-          buffers_.count(constants::delete_timestamps) != 0)
     , partial_tile_offsets_loading_(false) {
   // Sanity checks
-  if (storage_manager_ == nullptr) {
-    throw SparseIndexReaderBaseStatusException(
-        "Cannot initialize reader; Storage manager not set");
-  }
-
   if (!params.skip_checks_serialization() && buffers_.empty() &&
       aggregate_buffers_.empty()) {
-    throw SparseIndexReaderBaseStatusException(
+    throw SparseIndexReaderBaseException(
         "Cannot initialize reader; Buffers not set");
   }
 
@@ -97,7 +90,7 @@ SparseIndexReaderBase::SparseIndexReaderBase(
   offsets_format_mode_ =
       config_.get<std::string>("sm.var_offsets.mode", Config::must_find);
   if (offsets_format_mode_ != "bytes" && offsets_format_mode_ != "elements") {
-    throw SparseIndexReaderBaseStatusException(
+    throw SparseIndexReaderBaseException(
         "Cannot initialize reader; Unsupported offsets format in "
         "configuration");
   }
@@ -109,7 +102,7 @@ SparseIndexReaderBase::SparseIndexReaderBase(
   offsets_bitsize_ =
       config_.get<uint32_t>("sm.var_offsets.bitsize", Config::must_find);
   if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
-    throw SparseIndexReaderBaseStatusException(
+    throw SparseIndexReaderBaseException(
         "Cannot initialize reader; Unsupported offsets bitsize in "
         "configuration");
   }
@@ -146,108 +139,6 @@ uint64_t SparseIndexReaderBase::available_memory() {
          array_memory_tracker_->get_memory_usage();
 }
 
-std::vector<uint64_t> SparseIndexReaderBase::tile_offset_sizes() {
-  auto timer_se = stats_->start_timer("tile_offset_sizes");
-
-  // For easy reference.
-  std::vector<uint64_t> ret(fragment_metadata_.size());
-  const auto dim_num = array_schema_.dim_num();
-
-  // Compute the size of tile offsets per fragments.
-  const auto relevant_fragments = subarray_.relevant_fragments();
-  throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
-      0,
-      relevant_fragments.size(),
-      [&](uint64_t i) {
-        // For easy reference.
-        auto frag_idx = relevant_fragments[i];
-        auto& fragment = fragment_metadata_[frag_idx];
-        const auto& schema = fragment->array_schema();
-        const auto tile_num = fragment->tile_num();
-
-        // Compute the number of dimensions/attributes requiring offsets.
-        uint64_t num = 0;
-
-        // For fragments with version smaller than 5 we have zipped coords.
-        // Otherwise we load each dimensions independently.
-        if (fragment->version() < 5) {
-          num = 1;
-        } else {
-          for (unsigned d = 0; d < dim_num; ++d) {
-            // Fixed tile (offsets or fixed data).
-            num++;
-
-            // If var size, we load var offsets and var tile sizes.
-            if (is_dim_var_size_[d]) {
-              num += 2;
-            }
-          }
-        }
-
-        // Process everything loaded for query condition.
-        for (auto& name : qc_loaded_attr_names_) {
-          // Not a member of array schema, this field was added in array
-          // schema evolution, ignore for this fragment's tile offsets.
-          // Also skip dimensions.
-          if (!schema->is_field(name) || schema->is_dim(name)) {
-            continue;
-          }
-
-          // Fixed tile (offsets or fixed data).
-          num++;
-
-          // If var size, we load var offsets and var tile sizes.
-          const auto attr = schema->attribute(name);
-          num += 2 * attr->var_size();
-
-          // If nullable, we load nullable offsets.
-          num += attr->nullable();
-        }
-
-        // Process everything loaded for user requested data.
-        for (auto& it : buffers_) {
-          const auto& name = it.first;
-
-          // Skip dimensions and attributes loaded by query condition as they
-          // are processed above. Special attributes (timestamps, delete
-          // timestamps, etc.) are processed below.
-          if (array_schema_.is_dim(name) || !schema->is_field(name) ||
-              qc_loaded_attr_names_set_.count(name) != 0 ||
-              schema->is_special_attribute(name)) {
-            continue;
-          }
-
-          // Fixed tile (offsets or fixed data).
-          num++;
-
-          // If var size, we load var offsets and var tile sizes.
-          const auto attr = schema->attribute(name);
-          num += 2 * attr->var_size();
-
-          // If nullable, we load nullable offsets.
-          num += attr->nullable();
-        }
-
-        // Add timestamps if required.
-        if (!timestamps_not_present(constants::timestamps, frag_idx)) {
-          num++;
-        }
-
-        // Add delete metadata if required.
-        if (!delete_meta_not_present(constants::delete_timestamps, frag_idx)) {
-          num++;
-          num += deletes_consolidation_no_purge_;
-        }
-
-        // Finally set the size of the loaded data.
-        ret[frag_idx] = num * tile_num * sizeof(uint64_t);
-        return Status::Ok();
-      }));
-
-  return ret;
-}
-
 bool SparseIndexReaderBase::has_post_deduplication_conditions(
     FragmentMetadata& frag_meta) {
   return frag_meta.has_delete_meta() || condition_.has_value() ||
@@ -287,7 +178,8 @@ uint64_t SparseIndexReaderBase::get_coord_tiles_size(
       tiles_size += fragment_metadata_[f]->tile_size(dim_names_[d], t);
 
       if (is_dim_var_size_[d]) {
-        tiles_size += fragment_metadata_[f]->tile_var_size(dim_names_[d], t);
+        tiles_size += fragment_metadata_[f]->loaded_metadata()->tile_var_size(
+            dim_names_[d], t);
       }
     }
   }
@@ -325,10 +217,9 @@ Status SparseIndexReaderBase::load_initial_data() {
   const auto dim_num = array_schema_.dim_num();
 
   // Load delete conditions.
-  auto&& [st, conditions, update_values] =
-      storage_manager_->load_delete_and_update_conditions(*array_);
-  RETURN_CANCEL_OR_ERROR(st);
-  delete_and_update_conditions_ = std::move(*conditions);
+  auto&& [conditions, update_values] =
+      load_delete_and_update_conditions(resources_, *array_.get());
+  delete_and_update_conditions_ = conditions;
   bool make_timestamped_conditions = need_timestamped_conditions();
 
   if (make_timestamped_conditions) {
@@ -369,8 +260,11 @@ Status SparseIndexReaderBase::load_initial_data() {
   if (subarray_.is_set()) {
     // At this point, full memory budget is available.
     if (!array_memory_tracker_->set_budget(memory_budget_.total_budget())) {
-      throw SparseIndexReaderBaseStatusException(
-          "Cannot set array memory budget, already over limit.");
+      throw SparseIndexReaderBaseException(
+          "Cannot set array memory budget (" +
+          std::to_string(memory_budget_.total_budget()) +
+          ") because it is smaller than the current memory usage (" +
+          std::to_string(array_memory_tracker_->get_memory_usage()) + ").");
     }
 
     // Make sure there is no memory taken by the subarray.
@@ -380,9 +274,7 @@ Status SparseIndexReaderBase::load_initial_data() {
     // This is ok as it is a soft limit and will be taken into consideration
     // below.
     subarray_.precompute_all_ranges_tile_overlap(
-        storage_manager_->compute_tp(),
-        read_state_.frag_idx(),
-        &tmp_read_state_);
+        &resources_.compute_tp(), read_state_.frag_idx(), &tmp_read_state_);
 
     if (tmp_read_state_.memory_used_tile_ranges() >
         memory_budget_.ratio_tile_ranges() * memory_budget_.total_budget())
@@ -391,7 +283,7 @@ Status SparseIndexReaderBase::load_initial_data() {
   } else {
     for (const auto& [name, _] : aggregates_) {
       if (array_schema_.is_dim(name)) {
-        subarray_.load_relevant_fragment_rtrees(storage_manager_->compute_tp());
+        subarray_.load_relevant_fragment_rtrees(&resources_.compute_tp());
         break;
       }
     }
@@ -436,8 +328,12 @@ Status SparseIndexReaderBase::load_initial_data() {
   // Set a limit to the array memory.
   if (!array_memory_tracker_->set_budget(
           memory_budget_.total_budget() * memory_budget_.ratio_array_data())) {
-    throw SparseIndexReaderBaseStatusException(
-        "Cannot set array memory budget, already over limit.");
+    throw SparseIndexReaderBaseException(
+        "Cannot set array memory budget (" +
+        std::to_string(
+            memory_budget_.total_budget() * memory_budget_.ratio_array_data()) +
+        ") because it is smaller than the current memory usage (" +
+        std::to_string(array_memory_tracker_->get_memory_usage()) + ").");
   }
 
   // Add var size dimensions to the list of tile var size to load vector.
@@ -566,7 +462,7 @@ void SparseIndexReaderBase::compute_tile_bitmaps(
 
   // Compute parallelization parameters.
   uint64_t num_range_threads = 1;
-  const auto num_threads = storage_manager_->compute_tp()->concurrency_level();
+  const auto num_threads = resources_.compute_tp().concurrency_level();
   if (result_tiles.size() < num_threads) {
     // Ceil the division between thread_num and tile_num.
     num_range_threads = 1 + ((num_threads - 1) / result_tiles.size());
@@ -578,10 +474,7 @@ void SparseIndexReaderBase::compute_tile_bitmaps(
   if (num_range_threads != 1) {
     // Resize bitmaps to process for each tiles in parallel.
     throw_if_not_ok(parallel_for(
-        storage_manager_->compute_tp(),
-        0,
-        result_tiles.size(),
-        [&](uint64_t t) {
+        &resources_.compute_tp(), 0, result_tiles.size(), [&](uint64_t t) {
           static_cast<ResultTileWithBitmap<BitmapType>*>(result_tiles[t])
               ->alloc_bitmap();
           return Status::Ok();
@@ -590,7 +483,7 @@ void SparseIndexReaderBase::compute_tile_bitmaps(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_tiles.size(),
       0,
@@ -663,7 +556,7 @@ void SparseIndexReaderBase::compute_tile_bitmaps(
           {
             auto timer_compute_results_count_sparse =
                 stats_->start_timer("compute_results_count_sparse");
-            RETURN_NOT_OK(rt->compute_results_count_sparse(
+            throw_if_not_ok(rt->compute_results_count_sparse(
                 dim_idx,
                 ranges_for_dim,
                 relevant_ranges,
@@ -688,10 +581,7 @@ void SparseIndexReaderBase::compute_tile_bitmaps(
   if (num_range_threads != 1) {
     // Compute number of cells in each bitmaps in parallel.
     throw_if_not_ok(parallel_for(
-        storage_manager_->compute_tp(),
-        0,
-        result_tiles.size(),
-        [&](uint64_t t) {
+        &resources_.compute_tp(), 0, result_tiles.size(), [&](uint64_t t) {
           static_cast<ResultTileWithBitmap<BitmapType>*>(result_tiles[t])
               ->count_cells();
           return Status::Ok();
@@ -710,10 +600,7 @@ void SparseIndexReaderBase::apply_query_condition(
       use_timestamps_) {
     // Process all tiles in parallel.
     throw_if_not_ok(parallel_for(
-        storage_manager_->compute_tp(),
-        0,
-        result_tiles.size(),
-        [&](uint64_t t) {
+        &resources_.compute_tp(), 0, result_tiles.size(), [&](uint64_t t) {
           // For easy reference.
           auto rt = static_cast<ResultTileType*>(result_tiles[t]);
           const auto frag_meta = fragment_metadata_[rt->frag_idx()];
@@ -729,7 +616,7 @@ void SparseIndexReaderBase::apply_query_condition(
             // Remove cells with partial overlap from the bitmap.
             QueryCondition::Params params(
                 query_memory_tracker_, *(frag_meta->array_schema().get()));
-            RETURN_NOT_OK(partial_overlap_condition_.apply_sparse<BitmapType>(
+            throw_if_not_ok(partial_overlap_condition_.apply_sparse<BitmapType>(
                 params, *rt, rt->bitmap()));
             rt->count_cells();
           }
@@ -746,8 +633,9 @@ void SparseIndexReaderBase::apply_query_condition(
             // Remove cells deleted cells using the open timestamp.
             QueryCondition::Params params(
                 query_memory_tracker_, *(frag_meta->array_schema().get()));
-            RETURN_NOT_OK(delete_timestamps_condition_.apply_sparse<BitmapType>(
-                params, *rt, rt->post_dedup_bitmap()));
+            throw_if_not_ok(
+                delete_timestamps_condition_.apply_sparse<BitmapType>(
+                    params, *rt, rt->post_dedup_bitmap()));
             if (array_schema_.allows_dups()) {
               rt->count_cells();
             }
@@ -757,7 +645,7 @@ void SparseIndexReaderBase::apply_query_condition(
           if (condition_.has_value()) {
             QueryCondition::Params params(
                 query_memory_tracker_, *(frag_meta->array_schema().get()));
-            RETURN_NOT_OK(condition_->apply_sparse<BitmapType>(
+            throw_if_not_ok(condition_->apply_sparse<BitmapType>(
                 params, *rt, rt->post_dedup_bitmap()));
             if (array_schema_.allows_dups()) {
               rt->count_cells();
@@ -776,9 +664,10 @@ void SparseIndexReaderBase::apply_query_condition(
             for (uint64_t i = 0; i < delete_and_update_conditions_.size();
                  i++) {
               if (!frag_meta->has_delete_meta() ||
-                  frag_meta->get_processed_conditions_set().count(
-                      delete_and_update_conditions_[i].condition_marker()) ==
-                      0) {
+                  frag_meta->loaded_metadata()
+                          ->get_processed_conditions_set()
+                          .count(delete_and_update_conditions_[i]
+                                     .condition_marker()) == 0) {
                 auto delete_timestamp =
                     delete_and_update_conditions_[i].condition_timestamp();
 
@@ -791,12 +680,12 @@ void SparseIndexReaderBase::apply_query_condition(
                       *(frag_meta->array_schema().get()));
                   if (!frag_meta->has_timestamps() ||
                       delete_timestamp > frag_meta->timestamp_range().second) {
-                    RETURN_NOT_OK(
+                    throw_if_not_ok(
                         delete_and_update_conditions_[i]
                             .apply_sparse<BitmapType>(
                                 params, *rt, rt->post_dedup_bitmap()));
                   } else {
-                    RETURN_NOT_OK(
+                    throw_if_not_ok(
                         timestamped_delete_and_update_conditions_[i]
                             .apply_sparse<BitmapType>(
                                 params, *rt, rt->post_dedup_bitmap()));
@@ -1040,5 +929,4 @@ template void SparseIndexReaderBase::compute_tile_bitmaps<uint64_t>(
 template void SparseIndexReaderBase::compute_tile_bitmaps<uint8_t>(
     std::vector<ResultTile*>&);
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
