@@ -37,7 +37,6 @@
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/misc/tdb_math.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/storage_format/serialization/serializers.h"
 
 #include <cassert>
@@ -45,10 +44,17 @@
 #include <iostream>
 #include <list>
 
+/** Class for locally generated status exceptions. */
+class RTreeException : public StatusException {
+ public:
+  explicit RTreeException(const std::string& msg)
+      : StatusException("RTree", msg) {
+  }
+};
+
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 /* ****************************** */
 /*   CONSTRUCTORS & DESTRUCTORS   */
@@ -260,38 +266,34 @@ void RTree::serialize(Serializer& serializer) const {
   }
 }
 
-Status RTree::set_leaf(uint64_t leaf_id, const NDRange& mbr) {
+void RTree::set_leaf(uint64_t leaf_id, const NDRange& mbr) {
   if (levels_.size() != 1)
-    return LOG_STATUS(Status_RTreeError(
-        "Cannot set leaf; There are more than one levels in the tree"));
+    throw RTreeException(
+        "Cannot set leaf; There are more than one levels in the tree");
 
   if (leaf_id >= levels_[0].size())
-    return LOG_STATUS(Status_RTreeError("Cannot set leaf; Invalid lead index"));
+    throw RTreeException("Cannot set leaf; Invalid lead index");
 
   levels_[0][leaf_id] = mbr;
-
-  return Status::Ok();
 }
 
-Status RTree::set_leaves(const tdb::pmr::vector<NDRange>& mbrs) {
+void RTree::set_leaves(const tdb::pmr::vector<NDRange>& mbrs) {
   levels_.clear();
   levels_.resize(1);
   levels_[0].assign(mbrs.begin(), mbrs.end());
-  return Status::Ok();
 }
 
-Status RTree::set_leaf_num(uint64_t num) {
+void RTree::set_leaf_num(uint64_t num) {
   // There should be exactly one level (the leaf level)
   if (levels_.size() != 1)
     levels_.resize(1);
 
   if (num < levels_[0].size())
-    return LOG_STATUS(
-        Status_RTreeError("Cannot set number of leaves; provided number "
-                          "cannot be smaller than the current leaf number"));
+    throw RTreeException(
+        "Cannot set number of leaves; provided number "
+        "cannot be smaller than the current leaf number");
 
   levels_[0].resize(num);
-  return Status::Ok();
 }
 
 void RTree::deserialize(
@@ -345,17 +347,24 @@ void RTree::deserialize_v1_v4(
 
   auto level_num = deserializer.read<unsigned>();
   levels_.clear();
-  levels_.resize(level_num);
+  levels_.reserve(level_num);
   auto dim_num = domain->dim_num();
   for (unsigned l = 0; l < level_num; ++l) {
     auto mbr_num = deserializer.read<uint64_t>();
-    levels_[l].resize(mbr_num);
+    // We use emplace_back to construct the level in place, which also
+    // implicitly passes the memory tracking allocator all the way down to the
+    // NDRange objects.
+    auto& level = levels_.emplace_back();
+    level.reserve(mbr_num);
     for (uint64_t m = 0; m < mbr_num; ++m) {
-      levels_[l][m].resize(dim_num);
+      auto& mbr = level.emplace_back();
+      mbr.reserve(dim_num);
       for (unsigned d = 0; d < dim_num; ++d) {
         auto r_size{2 * domain->dimension_ptr(d)->coord_size()};
         auto data = deserializer.get_ptr<void>(r_size);
-        levels_[l][m][d] = Range(data, r_size);
+        // NDRange is not yet a pmr vector, so we need to explicitly pass the
+        // grandparent container's allocator.
+        mbr.emplace_back(data, r_size, level.get_allocator());
       }
     }
   }
@@ -370,27 +379,36 @@ void RTree::deserialize_v5(Deserializer& deserializer, const Domain* domain) {
   auto level_num = deserializer.read<unsigned>();
 
   levels_.clear();
-  levels_.resize(level_num);
+  levels_.reserve(level_num);
   auto dim_num = domain->dim_num();
 
   for (unsigned l = 0; l < level_num; ++l) {
     auto mbr_num = deserializer.read<uint64_t>();
-    levels_[l].resize(mbr_num);
+    // We use emplace_back to construct the level in place, which also
+    // implicitly passes the memory tracking allocator all the way down to the
+    // NDRange objects.
+    auto& level = levels_.emplace_back();
+    level.reserve(mbr_num);
 
     for (uint64_t m = 0; m < mbr_num; ++m) {
-      levels_[l][m].resize(dim_num);
+      auto& mbr = level.emplace_back();
+      mbr.reserve(dim_num);
       for (unsigned d = 0; d < dim_num; ++d) {
         auto dim{domain->dimension_ptr(d)};
         if (!dim->var_size()) {  // Fixed-sized
           auto r_size = 2 * dim->coord_size();
           auto data = deserializer.get_ptr<void>(r_size);
-          levels_[l][m][d] = Range(data, r_size);
+          // NDRange is not yet a pmr vector, so we need to explicitly pass the
+          // grandparent container's allocator.
+          mbr.emplace_back(data, r_size, level.get_allocator());
         } else {  // Var-sized
           // range_size | start_size | range
           auto r_size = deserializer.read<uint64_t>();
           auto start_size = deserializer.read<uint64_t>();
           auto data = deserializer.get_ptr<void>(r_size);
-          levels_[l][m][d] = Range(data, r_size, start_size);
+          // NDRange is not yet a pmr vector, so we need to explicitly pass the
+          // grandparent container's allocator.
+          mbr.emplace_back(data, r_size, start_size, level.get_allocator());
         }
       }
     }
@@ -399,5 +417,4 @@ void RTree::deserialize_v5(Deserializer& deserializer, const Domain* domain) {
   domain_ = domain;
 }
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
