@@ -111,6 +111,10 @@ class ContextResources {
     return rest_client_;
   }
 
+  [[nodiscard]] inline MemoryTrackerManager& memory_tracker_manager() const {
+    return *memory_tracker_manager_;
+  }
+
   /**
    * Create a new MemoryTracker
    *
@@ -131,6 +135,15 @@ class ContextResources {
    */
   shared_ptr<MemoryTracker> ephemeral_memory_tracker() const;
 
+  /**
+   * Return the serialization memory tracker.
+   *
+   * Use this tracker on serialization buffers.
+   *
+   * @return The serialization MemoryTracker.
+   */
+  shared_ptr<MemoryTracker> serialization_memory_tracker() const;
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
@@ -142,6 +155,9 @@ class ContextResources {
   /** The ephemeral MemoryTracker. */
   mutable shared_ptr<MemoryTracker> ephemeral_memory_tracker_;
 
+  /** The MemoryTracker for serialization operations. */
+  mutable shared_ptr<MemoryTracker> serialization_memory_tracker_;
+
   /** The MemoryTrackerReporter for this context. */
   mutable shared_ptr<MemoryTrackerReporter> memory_tracker_reporter_;
 
